<?php

/**
  @description This class provides all the functionality for an ia (interactive activities)
  @authors Patricia Alves Rodrigues and Leonidas O. Brandao
  @version $Id: locallib.php$ v 2.0$
  @package iAssign: Intective Assignments
  @last modification date 2012/10/14

  iMatica - iMath (http://www.matematica.br)
  Computer Science Dep. of IME-USP (Brazil)
 */
//require_once($CFG->libdir . '/formslib.php'); //putting this is as a safety as i got a class not found error.

/**
 * Standard base class for all iAssign.
 */
class iassign {

    var $cm;
    var $course;
    var $iassign;
    var $striassign;
    var $striassigns;
    var $context;
    var $activity;
    var $iassign_up;
    var $iassign_down;
    var $action;
    var $iassign_submission_current;
    var $userid_iassign;
    var $bottonPost;
    var $write_solution;
    var $view_iassign;

    /**
     * Constructor for the base iassign class
     */
    function iassign($iassign, $cm, $course) {
        global $COURSE, $CFG, $USER, $DB;

        $botton = optional_param('botton', NULL, PARAM_TEXT);
        if (!is_null($botton))
            $USER->iassignEdit = $botton;

        $this->userid_iassign = optional_param('userid_iassign', 0, PARAM_INT);
        $this->iassign_up = optional_param('iassign_up', 0, PARAM_INT);
        $this->iassign_down = optional_param('iassign_down', 0, PARAM_INT);
        $this->iassign_submission_current = optional_param('iassign_submission_current', 0, PARAM_INT);
        $this->write_solution = optional_param('write_solution', 0, PARAM_INT);
        $this->action = optional_param('action', NULL, PARAM_TEXT);

        if ($cm) {
            $this->cm = $cm;
        } else if (!$this->cm = get_coursemodule_from_id('iassign', $cmid)) {
            print_error('invalidcoursemodule');
        }

        $this->context = get_context_instance(CONTEXT_MODULE, $this->cm->id);

        if ($course) {
            $this->course = $course;
        } else if ($this->cm->course == $COURSE->id) {
            $this->course = $COURSE;
        } else if (!$this->course = $DB->get_record('course', array('id' => $this->cm->course))) {
            print_error('invalidid', 'iassign');
        }
        $this->coursecontext = get_context_instance(CONTEXT_COURSE, $this->course->id);
        $courseshortname = format_text($this->course->shortname, true, array('context' => $this->coursecontext));

        if ($iassign) {
            $this->iassign = $iassign;
        } else if (!$this->iassign = $DB->get_record('iassign', array('id' => $this->cm->instance))) {
            print_error('invalidid', 'iassign');
        }

        $USER->context = get_context_instance(CONTEXT_MODULE, $this->cm->id);
        $USER->cm = $this->cm->id;
        $this->iassign->cmidnumber = $this->cm->idnumber; // compatibility with modedit ia obj
        $this->iassign->courseid = $this->course->id; // compatibility with modedit ia obj
        $this->context = get_context_instance(CONTEXT_MODULE, $this->cm->id);
        $this->striassign = get_string('modulename', 'iassign');
        $this->striassigns = get_string('modulenameplural', 'iassign');
        $this->return = $CFG->wwwroot . "/mod/iassign/view.php?id=" . $this->cm->id;
        $this->bottonPost = 0;
        $this->view_iassign = optional_param('action', false, PARAM_BOOL);
        $this->activity = new activity(optional_param('iassign_current', NULL, PARAM_TEXT));
        $this->view();
    }

    /**
     * Display the iAssign, used by view.php
     *
     * This in turn calls the methods producing individual parts of the page
     */
    function view() {
        global $USER, $DB, $OUTPUT;

        // If this user has no capability to View 'iassign': stop here
        require_capability('mod/iassign:view', $this->context);

        add_to_log($this->course->id, "iassign", "view", "view.php?id={$this->cm->id}", $this->iassign->name, $this->cm->id, $USER->id);

        if ($this->action) {
            $this->action();
        } else {
            echo $OUTPUT->header();
            $this->view_iassigns();
            echo $OUTPUT->footer();
        } //if ($this->action)
        //security: delete all records with an error loading IMA
        $DB->delete_records("iassign_security", array("userid" => $USER->id, "view" => 1));
        die;
    }

    /**
     * Limited access
     */
    function action() {
        global $USER;

        //action:
        //up - move up activity (mover atividade para cima)
        //down - move down activity (mover atividade para baixo)
        //visible - view/hide activity (exibir/ocultar atividade)
        //delete - delete activity (excluir atividade)
        //deleteyes - confirms exclusion of activity (confirma exclusão de atividade)
        //deleteno - does not erase activity (não apaga atividade)
        //add - add activity (adicionar atividade)
        //edit - edit activity (modificar atividade)

        $action_iassign = array(
            'newcomment' => '$this->get_answer ();',
            'view' => '$this->view_iassign_current ();',
            'get_answer' => '$this->get_answer ();',
            'repeat' => '$this->view_iassign_current ();',
            'overwrite' => '$this->get_answer ();',
            'stats_student' => '$this->stats_students ();',
        );

        $action_iassign_limit = array(
            'view' => '$this->view_iassign_current();',
            'newcomment' => '$this->get_answer ();',
            'viewsubmission' => '$this->view_iassign_current();',
            'edit_status' => '$this->edit_status ();',
            'edit_grade' => '$this->edit_grade ();',
            'report' => '$this->report ();',
            'print' => '$this->report ();',
            'stats' => '$this->stats ();',
            'printstats' => '$this->stats ();'
        );

        $restricted = array(
            'up' => '$this->activity->move_iassign ($this->iassign_up,$this->return);',
            'down' => '$this->activity->move_iassign ($this->iassign_down,$this->return);',
            'visible' => '$this->activity->visible_iassign ($this->return);',
            'delete' => '$this->activity->delete ($this->return);',
            'deleteno' => '$this->activity->deleteno($this->return);',
            'deleteyes' => '$this->activity->deleteyes($this->return);',
            'add' => '$this->add_edit_iassign ();',
            'edit' => '$this->add_edit_iassign ();',
            'get_answer' => '$this->get_answer ();',
        );

        $action_iassign_restricted = array_merge($restricted, $action_iassign_limit, $action_iassign);

        if (has_capability('mod/iassign:editiassign', $this->context, $USER->id))
            eval($action_iassign_restricted[$this->action]);
        elseif (has_capability('mod/iassign:evaluateiassign', $this->context, $USER->id))
            eval($action_iassign_limit[$this->action]);
        else
            eval($action_iassign[$this->action]);
    }

    /**
     * receives the return of iLM
     */
    function get_answer() {
        global $USER, $CFG, $DB, $OUTPUT;
        $id = $this->cm->id;
        $submission_comment = optional_param('submission_comment', NULL, PARAM_TEXT);
        $comment = false;
        if ($submission_comment)
            $comment = $this->write_comment_submission();

        //receives data of iLM using the current activity
        $iassign_ilm = $DB->get_record("iassign_ilm", array("id" => $this->activity->get_activity()->iassign_ilmid));
        $iassign = $DB->get_record("iassign", array("id" => $this->activity->get_activity()->iassignid));
        //receives data of submission of current activity
        $iassign_submission = $DB->get_record("iassign_submission", array("iassign_statementid" => $this->activity->get_activity()->id, "userid" => $this->userid_iassign)); // data about student solution
        //receives post and get
        $MA_POST_Value = optional_param('MA_POST_Value', 0, PARAM_INT); // 1 - activity evaluated as correct  / 0 -  activity evaluated as incorrect
        $MA_POST_Archive = optional_param('MA_POST_Archive', NULL, PARAM_TEXT); //answer file (string)
        $MA_POST_Info = optional_param('MA_POST_Info', NULL, PARAM_FORMAT);
        $MA_POST_SystemData = optional_param('MA_POST_SystemData', NULL, PARAM_FORMAT);
        $return_get_answer = optional_param('return_get_answer', 0, PARAM_INT);
        $msg = '';

        //feedback
        /*
          status of activities:
          0 – not post
          1 – post
          2 – evaluated as incorrect
          3 – evaluated as correct
         */

        $title = get_string('evaluate_iassign', 'iassign');
        echo $OUTPUT->header();
        echo $OUTPUT->box_start();
        $return = "" . $CFG->wwwroot . "/mod/iassign/view.php?action=view&id=" . $id . "&iassign_submission_current=" . $this->iassign_submission_current . "&userid_iassign=" . $this->userid_iassign . "&iassign_current=" . $this->activity->get_activity()->id;

        // icons::insert ($icon)
        // $return_last = "&nbsp;<a href='" . $return . "'>" . $this->icon_return . '&nbsp;' . get_string('return_iassign', 'iassign') . "</a>";
        $return_last = "&nbsp;<a href='" . $return . "'>" . icons::insert('return_home') . '&nbsp;' . get_string('return_iassign', 'iassign') . "</a>";

        $link_return = "&nbsp;<a href='" . $this->return . "'>" . icons::insert('home') . '&nbsp;' . get_string('activities_page', 'iassign') . "</a>";
        echo '<table  width=100% >';

        if ($MA_POST_Archive == -1 or empty($MA_POST_Archive)) { // if ($MA_POST_Value == -1) {
            $this->write_solution = 0; // necessary in order to take note in Moodle 'grade' system
            if ($comment)
                echo '<tr><td colspan=2><br>' . get_string('empty_answer_post', 'iassign') . '</br>'
                . get_string('confirm_add_comment', 'iassign') . '</td>';
            else
                echo '<tr><td colspan=2><br>' . get_string('empty_answer_post', 'iassign') . '</td>';
            echo '<tr><td width=40% align=right>' . $return_last . '&nbsp;' . $link_return . '</td></tr>';
        } else {
            if ($iassign_ilm->evaluate == 1 and $this->activity->get_activity()->automatic_evaluate == 1) { // iLM with automatic evaluator
                if (intval($MA_POST_Value) == 1) {
                    $status = 3;
                    $grade_student = $this->activity->get_activity()->grade; // evaluated as correct solution submitted is assigned the note pattern of activity
                    $msg = '<tr><td colspan=2>' . icons::insert('feedback_correct') . '<br>' . get_string('get_answer_correct', 'iassign') . '</td>';

                    //log record
                    $info = $iassign->name . "&nbsp;-&nbsp;" . $this->activity->get_activity()->name . "&nbsp;-&nbsp;" . get_string('feedback_correct', 'iassign') . "&nbsp;-&nbsp;" . get_string('grade_iassign', 'iassign') . ":" . $grade_student;
                    add_to_log($this->course->id, "iassign", "add submission", "view.php?id={$this->cm->id}", $info, $this->cm->id, $USER->id);
                } else {
                    $status = 2;
                    $grade_student = 0; // evaluated as incorrect solution
                    $msg = '<tr><td colspan=2>' . icons::insert('feedback_incorrect') . '<br>' . get_string('get_answer_incorrect', 'iassign') . '</td>';

                    //log record
                    $info = $iassign->name . "&nbsp;-&nbsp;" . $this->activity->get_activity()->name . "&nbsp;-&nbsp;" . get_string('feedback_incorrect', 'iassign') . '&nbsp;-&nbsp;' . get_string('grade_iassign', 'iassign') . $grade_student;
                    add_to_log($this->course->id, "iassign", "add submission", "view.php?id={$this->cm->id}", $info, $this->cm->id, $USER->id);
                } // if ($MA_POST_Value == 1)

                if ($this->activity->get_activity()->show_answer == 0) { //not automatic evaluate
                    echo '<tr><td width=60% ><strong>' . icons::insert('post') . get_string('get_answer', 'iassign') . '</strong></td>';
                    echo '<tr><td width=40% align=right>' . $return_last . '&nbsp;' . $link_return . '</td></tr>';
                    echo '<tr>';
                    //log record
                    $info = $iassign->name . "&nbsp;-&nbsp;" . $this->activity->get_activity()->name . "&nbsp;-&nbsp;" . get_string('get_answer', 'iassign');
                    add_to_log($this->course->id, "iassign", "add submission", "view.php?id={$this->cm->id}", $info, $this->cm->id, $USER->id);
                } else {
                    echo '<tr><td width=60% ><strong>' . get_string('auto_result', 'iassign') . '</strong></td>';
                    echo '<td width=40% align=right>' . $return_last . '&nbsp;' . $link_return . '</td></tr>';
                    echo '<tr>';
                    echo $msg;
                }
            } else {
                $status = 1;
                $grade_student = 0; // iLM not have automatic evaluator
                echo '<tr><td colspan=2>' . icons::insert('post') . get_string('get_answer_post', 'iassign') . '</td>';
                echo '<tr><td width=40% align=right>' . $return_last . '&nbsp;' . $link_return . '</td></tr>';
                echo '<tr>';

                //log record
                $info = $iassign->name . "&nbsp;-&nbsp;" . $this->activity->get_activity()->name . "&nbsp;-&nbsp;" . get_string('get_answer_post', 'iassign');
                add_to_log($this->course->id, "iassign", "add submission", "view.php?id={$this->cm->id}", $info, $this->cm->id, $USER->id);
            } //if ($iassign_ilm->evaluate == 1)
        } //if ($MA_POST_Value == -1)
        echo '</tr></table>';
        echo $OUTPUT->box_end();

        // add or update evaluate
        if ($this->write_solution == 1) {
            $timenow = time();

            //new record
            if (!$iassign_submission) {
                $newentry = new stdClass();
                $newentry->userid = $this->userid_iassign;
                $newentry->iassign_statementid = $this->activity->get_activity()->id;
                $newentry->timecreated = $timenow;
                $newentry->timemodified = $timenow;
                $newentry->answer = $MA_POST_Archive;
                $newentry->grade = $grade_student;
                $newentry->status = $status;
                $newentry->experiment = 1;
                if (!$newentry->id = $DB->insert_record("iassign_submission", $newentry)) {
                    print_error('error_insert', 'iassign');
                } else {
                    add_to_log($this->course->id, "iassign", "add submission", "view.php?id={$this->iassign->id}", $this->activity->get_activity()->name, $this->cm->id, $USER->id);
                    $this->update_grade_student($newentry->userid, $newentry->iassign_statementid, $this->iassign->id);
                }
            } elseif ($iassign_submission->status != 3) {
                $newentry = new stdClass();
                $newentry->id = $iassign_submission->id;
                $newentry->iassign_statementid = $iassign_submission->iassign_statementid;
                $newentry->userid = $iassign_submission->userid;
                $newentry->timecreated = $iassign_submission->timecreated;
                $newentry->timemodified = $timenow;
                $newentry->answer = $MA_POST_Archive;
                $newentry->grade = $grade_student;
                $newentry->status = $status;
                $newentry->experiment = $iassign_submission->experiment + 1;
                if (!$DB->update_record("iassign_submission", $newentry)) {
                    print_error('error_update', 'iassign');
                    //D depurar...
                    //D $stringAux = "ia.class.php: ".$MA_POST_Archive."<br/> ".utf8_encode($MA_POST_Archive)."<br/>".utf8_encode(utf8_encode($MA_POST_Archive))."<br/>";
                    //D $fp = fopen("teste1.txt","w");
                    //D fwrite($fp,$stringAux);
                } else {
                    add_to_log($this->course->id, "iassign", "update submission", "view.php?id={$this->iassign->id}", $this->activity->get_activity()->name, $this->cm->id, $USER->id);
                    $this->update_grade_student($newentry->userid, $newentry->iassign_statementid, $this->iassign->id);
                }
            } else {
                if ($return_get_answer == 1) {
                    $newentry = new stdClass();
                    $newentry->id = $iassign_submission->id;
                    $newentry->iassign_statementid = $iassign_submission->iassign_statementid;
                    $newentry->userid = $iassign_submission->userid;
                    $newentry->timecreated = $iassign_submission->timecreated;
                    $newentry->timemodified = $timenow;
                    $newentry->answer = $MA_POST_Archive;
                    $newentry->grade = $grade_student;
                    $newentry->status = $status;
                    $newentry->experiment = $iassign_submission->experiment + 1;
                    if (!$DB->update_record("iassign_submission", $newentry))
                        print_error('error_update', 'iassign');
                    else {
                        add_to_log($this->course->id, "iassign", "update submission", "view.php?id={$this->iassign->id}", $this->activity->get_activity()->name, $this->cm->id, $USER->id);
                        $this->update_grade_student($newentry->userid, $newentry->iassign_statementid, $this->iassign->id);
                        echo $OUTPUT->box_start();
                        echo "<p>" . get_string('iassign_update', 'iassign') . "</p>";
                        echo $OUTPUT->box_end();
                    }
                } elseif ($return_get_answer == 2) {
                    echo $OUTPUT->box_start();
                    echo "<p>" . get_string('iassign_cancel', 'iassign') . "</p>";
                    echo $OUTPUT->box_end();
                } else {
                    echo $OUTPUT->box_start();
                    echo "
         <script type='text/javascript'>
             //<![CDATA[
          function overwrite () {
           document.formEnvio.return_get_answer.value = 1;
           document.formEnvio.submit();
          }

          function nooverwrite () {
           document.formEnvio.return_get_answer.value = 2;
           document.formEnvio.submit();
          }
          //]]>
         </script>";
                    $param_aux = "action=overwrite&iassign_submission_current=" . $iassign_submission->id . "&id=" . $id . "&iassign_current=" . $this->activity->get_activity()->id . "&write_solution=" . $this->write_solution . "&userid_iassign=" . $USER->id;
                    $get_answer_overwrite = $CFG->wwwroot . "/mod/iassign/view.php?" . $param_aux;
                    echo "<form name='formEnvio' method='post' action='$get_answer_overwrite' enctype='multipart/form-data'>";
                    echo "<p>" . get_string('last_iassign_correct', 'iassign') . "</p>";
                    echo "<p>" . get_string('update_iassign', 'iassign') . "</p>";
                    echo "<input type='hidden' name='MA_POST_Archive' value='$MA_POST_Archive'/>
           <input type='hidden' name='MA_POST_Value' value='$MA_POST_Value'/>
           <input type='hidden' name='MA_POST_Info' value='$MA_POST_Info'/>
           <input type='hidden' name='MA_POST_SystemData' value='$MA_POST_SystemData'/>
           <input type='hidden' name='return_get_answer'/> ";
                    echo "<input type=button value='" . get_string('yes', 'iassign') . "' onClick = 'overwrite()'
            title='" . get_string('message_update_iassign', 'iassign') . "'/>&nbsp;&nbsp;";
                    echo "<input type=button value='" . get_string('no', 'iassign') . "' onClick = 'nooverwrite()'
            title='" . get_string('message_no_update_iassign', 'iassign') . "'/>";
                    echo" </form>";
                    echo $OUTPUT->box_end();
                }
            }
        } //if ($this->write_solution == 1)

        echo $OUTPUT->footer();
        die;
    }

    /**
     * Editing status of interactive activities
     */
    function edit_status() {
        global $USER, $DB, $OUTPUT;

        $newentry->id = $this->iassign_submission_current;
        $newentry->status = optional_param('return_status', 0, PARAM_INT);

        $iassign_submission = $DB->get_record('iassign_submission', array('id' => $this->iassign_submission_current));
        if ($iassign_submission->status != 0 and $newentry->status == 0)
            $newentry->status = $iassign_submission->status;

        $newentry->teacher = $USER->id;
        if (!$DB->update_record('iassign_submission', $newentry))
            print_error('error_update', 'iassign');
        else {
            add_to_log($this->course->id, "iassign", "update", "view.php?id={$this->iassign->id}", $newentry->id, $this->cm->id, $USER->id);
            $this->action = 'viewsubmission';
            $this->view_iassign_current();
        } //if (!$DB->update_record('iassign_submission', $newentry))
    }

    /**
     * Editing grade of interactive activities
     */
    function edit_grade() {
        global $USER, $DB, $OUTPUT;

        $newgrade = optional_param('return_grade', 0, PARAM_INT);
        if ($newgrade and $newgrade >= 0) {
            $newentry->id = $this->iassign_submission_current;
            $newentry->grade = optional_param('return_grade', 0, PARAM_INT);
            $newentry->teacher = $USER->id;
            if (!$DB->update_record('iassign_submission', $newentry))
                print_error('error_update', 'iassign');
            else
                add_to_log($this->course->id, "iassign", "update", "view.php?id={$this->iassign->id}", $newentry->id, $this->cm->id, $USER->id);
        } //if ($newgrade >= 0)
        $this->action = 'viewsubmission';
        $this->view_iassign_current();
    }

    /**
     * Add or Edit interactive activities
     */
    function add_edit_iassign() {
        global $USER, $CFG, $COURSE, $DB, $OUTPUT;
        require_once('iassign_form.php');

        $id = $this->cm->id;
        $iassignid = $this->iassign->id;

        $param = new object();
        $param->action = $this->action; // oculto
        $param->id = $id; // oculto
        $COURSE->cm = $id;
        $COURSE->iassignid = $iassignid;

        $context = get_context_instance(CONTEXT_MODULE, $this->cm->id);

        $contextuser = get_context_instance(CONTEXT_USER, $USER->id);

        $component = 'mod_iassign';
        $filearea = 'activity';

        if (!empty($this->iassign_current))
            $COURSE->iassign_id = $this->iassign_current;
        else
            $COURSE->iassign_id = 0;

        if ($this->action == 'add') {

            $params = array('iassignid' => $iassignid);
            $iassign_statement = $DB->get_records_sql("SELECT s.id, s.name, s.dependency
                             FROM {iassign_statement} s
                             WHERE s.iassignid = :iassignid
                             ORDER BY `s`.`position` ASC", $params); // " - jed/excs

            $param->iassignid = $iassignid;
            $param->name = "";
            $param->oldname = "";
            $param->type_iassign = 3;
            $param->proposition = "";
            $author = $DB->get_record("user", array("id" => $USER->id));
            $param->author_name = $author->firstname . '&nbsp;' . $author->lastname;
            $param->author_modified_name = $author->firstname . '&nbsp;' . $author->lastname;
            $param->author = $param->author_name;
            $param->author_modified = $param->author_modified_name;
            $COURSE->iassign_list = array();
            $param->iassign_list = array();
            if ($iassign_statement) {
                foreach ($iassign_statement as $iassign) {
                    $param->iassign_list[$iassign->id] = 0;
                    $COURSE->iassign_list[$iassign->id] = new stdClass();
                    $COURSE->iassign_list[$iassign->id]->id = $iassign->id;
                    $COURSE->iassign_list[$iassign->id]->name = $iassign->name;
                    $COURSE->iassign_list[$iassign->id]->enable = 1;
                } //foreach ($iassign_statement as $iassign)
            }
            $param->iassign_ilmid = 0;
            $param->file = 0;
            $param->fileold = 0;
            $param->filename = "";
            $param->grade = 100;
            $param->timemodified = time();
            $param->timecreated = time();
            $param->timeavailable = time();
            $param->timedue = time() + 7 * 24 * 3600;
            $param->preventlate = 0;
            $param->test = 0;
            $param->special_param1 = 0;
            $param->visible = 1;
            $param->max_experiment = 0;
            $param->dependency = 0;
            $param->automatic_evaluate = 1;
            $param->show_answer = 1;
        }  // if ($this->action == 'add')
        elseif ($this->action == 'edit') {

            $COURSE->iassign_list = array();

            if ($this->activity->get_activity() != null) {
                $iassign_statement_current = $DB->get_record("iassign_statement", array("id" => $this->activity->get_activity()->id));


                $iassign_statement = $DB->get_records_sql("SELECT *
                             FROM {$CFG->prefix}iassign_statement s
                             WHERE s.iassignid = '$iassignid'
                             AND s.id!='$iassign_statement_current->id'
                             ORDER BY `s`.`position` ASC");

                $param->iassign_id = $iassign_statement_current->id; // oculto
                $param->iassignid = $iassign_statement_current->iassignid; // oculto
                $param->name = $iassign_statement_current->name;
                $param->oldname = $iassign_statement_current->name;
                $param->type_iassign = $iassign_statement_current->type_iassign;
                $param->proposition = $iassign_statement_current->proposition;
                $param->author_name = $iassign_statement_current->author_name; // oculto
                $param->author = $iassign_statement_current->author_name;
                $author = $DB->get_record("user", array('id' => $USER->id));
                $param->author_modified_name = $author->firstname . '&nbsp;' . $author->lastname;
                $param->author_modified = $param->author_modified_name;
                $dependency = explode(';', $iassign_statement_current->dependency);
                $param->iassign_list = array();
                $inter = array();
                if ($iassign_statement)
                    foreach ($iassign_statement as $iassign)
                        if (in_array($iassign->id, $dependency))
                            $inter[] = $iassign->id;

                $iassign_statement_dependency = $DB->get_records_sql("SELECT *
                             FROM {$CFG->prefix}iassign_statement s
                             WHERE s.iassignid = '$iassignid'
                             AND s.id!='$iassign_statement_current->id'
                             AND s.dependency!=0");

                $array_dependency = array();
                $subdependency = "";
                $sub_subdependency = "";
                //dependent on this exercise
                if ($iassign_statement_dependency) {
                    $subdependency .= $this->search_dependency($iassign_statement_current->id, $iassign_statement_dependency);

                    //to whom this exercise depends
                    foreach ($inter as $tmp)
                        $sub_subdependency .=$this->search_sub_dependency($tmp);

                    $list_dependency = $subdependency . $sub_subdependency;
                    $array_dependency = explode(";", $list_dependency);
                }

                if ($iassign_statement) {
                    foreach ($iassign_statement as $iassign) {

                        $COURSE->iassign_list[$iassign->id] = new stdClass();
                        $COURSE->iassign_list[$iassign->id]->name = $iassign->name;
                        $COURSE->iassign_list[$iassign->id]->id = $iassign->id;

                        if (in_array($iassign->id, $dependency))
                            $param->iassign_list[$iassign->id] = 1;
                        else
                            $param->iassign_list[$iassign->id] = 0;

                        if (in_array($iassign->id, $array_dependency))
                            $COURSE->iassign_list[$iassign->id]->enable = 0;
                        else
                            $COURSE->iassign_list[$iassign->id]->enable = 1;
                    } // foreach ($iassign_statement as $iassign)
                }
                $param->iassign_ilmid = $iassign_statement_current->iassign_ilmid;
                $param->fileold = 0;
                $param->file = 0;
                $param->filename = '';
                $param->grade = $iassign_statement_current->grade;
                $param->timecreated = $iassign_statement_current->timecreated; // oculto
                $param->timeavailable = $iassign_statement_current->timeavailable;
                $param->timedue = $iassign_statement_current->timedue;
                $param->preventlate = $iassign_statement_current->preventlate;
                $param->test = $iassign_statement_current->test;
                $param->special_param1 = $iassign_statement_current->special_param1;
                $param->position = $iassign_statement_current->position; // oculto
                $param->visible = $iassign_statement_current->visible;
                $param->max_experiment = $iassign_statement_current->max_experiment;
                $param->automatic_evaluate = $iassign_statement_current->automatic_evaluate;
                $param->show_answer = $iassign_statement_current->show_answer;
                $fs = get_file_storage();

                $files = $fs->get_area_files($context->id, $component, $filearea, $iassign_statement_current->file);
                if ($files) {
                    foreach ($files as $file) {
                        if ($file->get_filename() != '.') {
                            $param->filename = $file->get_filename();
                            $param->file = $file->get_id();
                            $param->fileold = $file->get_id();
                        }
                    }
                }
            } //elseif ($this->action == 'edit')
        }
        //search position
        $iassign_list = $DB->get_records_list('iassign_statement', 'iassignid', array($this->iassign->id), 'position ASC');

        if ($iassign_list) {
            $end_list = array_pop($iassign_list);
            $param->position = $end_list->position + 1;
        } //if ($iassign_list)
        else
            $param->position = 1;

        echo $OUTPUT->header();

        $mform = new mod_iassign_form();
        $mform->set_data($param);

        if ($mform->is_cancelled()) {
            $this->return_home_course('iassign_cancel');
            die();
        } else if ($result = $mform->get_data()) {

            $result->context = $context;

            if ($result->type_iassign == 1 or $result->type_iassign == 2)
                $result->grade = 0;
            if ($result->type_iassign == 1) {
                $result->automatic_evaluate = 0;
                $result->show_answer = 0;
            } elseif ($result->automatic_evaluate == 0)
                $result->show_answer = 0;

            // $result->iassign_list = $_POST['iassign_list'];
            // $result_assign_list = $_GET['iassign_list'];

            $result->iassign_list = optional_param_array('iassign_list', array(), PARAM_RAW);
            if ($result->iassign_list) {
                foreach ($result->iassign_list as $key => $value)
                    $result->dependency.=$key . ';';
            } //if ($result_assign_list)
            else
                $result->dependency = 0;

            $iassign_ilm = $DB->get_record("iassign_ilm", array("id" => $result->iassign_ilmid));

            if ($this->action == 'add') {
                $iassign_statement_name = $DB->get_records_sql("SELECT *
                             FROM {$CFG->prefix}iassign_statement s
                             WHERE s.iassignid = '$result->iassignid'
                             AND s.name = '$result->name'");
                if ($iassign_statement_name) {
                    $this->return_home_course('error_iassign_name');
                    die;
                }

                $iassignid = $this->activity->new_iassign($result);

                $this->activity->add_calendar($iassignid);
                add_to_log($this->course->id, "iassign", "add", "view.php?id=$id", $result->name, $this->cm->id, $USER->id);
                $this->return_home_course('iassign_add');
            } //if ($this->action == 'add')
            elseif ($this->action == 'edit') {

                $iassignid = $this->activity->update_iassign($result);
                $this->activity->update_calendar($iassignid, $result->oldname);

                add_to_log($this->course->id, "iassign", "update", "view.php?id=$id", $result->name, $this->cm->id, $USER->id);
                $this->return_home_course('iassign_update');
            } //elseif ($this->action == 'edit')
            die();
        } //elseif ($mform->get_data())

        $mform->display();

        echo $OUTPUT->footer();
        die;
    }

    /**
     * Search dependency
     */
    function search_dependency($search_iassing_id, $iassign_statement) {
        global $DB, $OUTPUT;
        $dependency = "";
        if ($iassign_statement)
            foreach ($iassign_statement as $iassign) {
                $inter_dependency = explode(';', $iassign->dependency);
                if (in_array($search_iassing_id, $inter_dependency)) {
                    $dependency .=$iassign->id . ";";
                    $dependency .= $this->search_dependency($iassign->id, $iassign_statement);
                } // if (in_array($search_iassing_id, $inter_dependency))
            } //foreach ($iassign_statement as $iassign)
        return $dependency;
    }

    /**
     * Search subdependency
     */
    function search_sub_dependency($search_iassing_id) {
        global $DB, $OUTPUT;

        // $iassign_statement = $DB->get_record('iassign_statement', 'id', $search_iassing_id);
        $iassign_statement = $DB->get_record("iassign_statement", array("id" => $search_iassing_id));

        $dependency = "";
        if ($iassign_statement) {
            $inter_dependency = explode(';', $iassign_statement->dependency);

            foreach ($inter_dependency as $tmp) {
                if ($tmp != 0)
                    $dependency .=$tmp . ";";
                $dependency .= $this->search_sub_dependency($tmp);
            } //foreach ($inter_dependency as $tmp)
        } //if ($iassign_statement)
        return $dependency;
    }

    /**
     * Update grade of iAssign
     */
    static function update_grade_iassign($iassignid) {
        global $USER, $CFG, $COURSE, $DB, $OUTPUT;
        require_once($CFG->libdir . '/gradelib.php');
        $sum_grade = $DB->get_records_sql("SELECT SUM(grade) as total
                             FROM {$CFG->prefix}iassign_statement s
                             WHERE s.iassignid = '$iassignid'
                             AND s.type_iassign=3");

        $grade_iassign = $DB->get_record("iassign", array("id" => $iassignid));
        $grades = NULL;
        $params = array('itemname' => $grade_iassign->name);
        $params['iteminstance'] = $iassignid;
        $params['gradetype'] = GRADE_TYPE_VALUE;
        if (key($sum_grade)) {
            $params['grademax'] = key($sum_grade);
            $params['rawgrademax'] = key($sum_grade);
        } else {
            $params['grademax'] = 0;
            $params['rawgrademax'] = 0;
        }
        $params['grademin'] = 0;
        grade_update('mod/iassign', $grade_iassign->course, 'mod', 'iassign', $iassignid, 0, $grades, $params);
    }

    /**
     * Update grade of student
     */
    function update_grade_student($userid, $iassign_statementid, $iassignid) {
        global $CFG, $DB, $OUTPUT;
        require_once($CFG->libdir . '/gradelib.php');
        $grade_iassign = $DB->get_record('iassign', array('id' => $iassignid));
        $grade_iassign_statements = $DB->get_records('iassign_statement', array('iassignid' => $iassignid));
        $total_grade = 0;

        foreach ($grade_iassign_statements as $grade_iassign_statement) {
            $iassign_submission = $DB->get_record('iassign_submission', array('iassign_statementid' => $grade_iassign_statement->id, 'userid' => $userid));
            if ($iassign_submission)
                $total_grade += $iassign_submission->grade;
        } //foreach ($grade_iassign_statements as $grade_iassign_statement)
        $sum_grade = $DB->get_records_sql("SELECT SUM(grade) as total
                             FROM {$CFG->prefix}iassign_statement s
                             WHERE s.iassignid = '$iassignid'
                             AND s.type_iassign=3");

        $grades['userid'] = $userid;
        $grades['rawgrade'] = $total_grade;
        $params = array('itemname' => $grade_iassign->name);
        $params['iteminstance'] = $iassignid;
        $params['gradetype'] = GRADE_TYPE_VALUE;

        if (key($sum_grade)) {
            $params['grademax'] = key($sum_grade);
            $params['rawgrademax'] = key($sum_grade);
        } else {
            $params['grademax'] = 0;
            $params['rawgrademax'] = 0;
        }
        grade_update('mod/iassign', $grade_iassign->course, 'mod', 'iassign', $iassignid, 0, $grades, $params);
    }

    /**
     * Display caption of icons
     */
    function view_legend_icons() {

        global $USER, $CFG, $DB, $OUTPUT;
        $id = $this->cm->id;

        if ($this->action == 'print')
            echo '<table border=1 width=100%><tr>';
        else
            echo '<table width=100%><tr>';
        echo '<td >';
        if ($this->action != 'print')
            echo $OUTPUT->help_icon('legend', 'iassign');

        // helpbutton('legend', get_string('legend', 'iassign'), 'iassign', $image = true, $linktext = false, $text = '', $return = false,
        // $imagetext = '');
        echo '<strong>' . get_string('legend', 'iassign') . '</strong>';
        echo '&nbsp;' . icons::insert('correct') . '&nbsp;' . get_string('correct', 'iassign');
        echo '&nbsp;' . icons::insert('incorrect') . '&nbsp;' . get_string('incorrect', 'iassign');
        echo '&nbsp;' . icons::insert('post') . '&nbsp;' . get_string('post', 'iassign');
        echo '&nbsp;' . icons::insert('not_post') . '&nbsp;' . get_string('not_post', 'iassign');
        echo '&nbsp;' . icons::insert('comment_unread') . '&nbsp;' . get_string('comment_unread', 'iassign');

        if (has_capability('mod/iassign:viewreport', $this->context, $USER->id) && $this->action == 'report') {
            echo '&nbsp;' . icons::insert('comment_read') . '&nbsp;' . get_string('comment_read', 'iassign');
            echo '</td>' . "\n";
            if ($this->action != 'print') {
                $link_print = "<a href='" . $CFG->wwwroot . "/mod/iassign/view.php?id=" . $id . "&action=print&iassignid=" . $this->iassign->id . "'>" . icons::insert('print') . '&nbsp;' . get_string('print', 'iassign') . "</a>";
                $link_stats = "<a href='" . $CFG->wwwroot . "/mod/iassign/view.php?id=" . $id . "&action=stats&iassignid=" . $this->iassign->id . "'>" . icons::insert('results') . '&nbsp;' . get_string('graphic', 'iassign') . "</a>";
                echo'<td width=15% align="right">' . $link_stats . '</td>' . "\n";
                echo'<td width=15% align="right">' . $link_print . '</td>' . "\n";
            } //if ($this->action != 'print')
            echo '</tr></table>' . "\n";
        }  //if (has_capability('mod/iassign:viewreport', $this->context, $USER->id) and $this->action == 'report')
        elseif (has_capability('mod/iassign:submitiassign', $this->context, $USER->id)) {
            $link_stats = "<a href='" . $CFG->wwwroot . "/mod/iassign/view.php?id=" . $id . "&action=stats_student&iassignid=" . $this->iassign->id . "'>" . icons::insert('results') . '&nbsp;' . get_string('results', 'iassign') . "</a>";
            echo'<td width=15% align="right">' . $link_stats . '</td>' . "\n";
            echo '</tr></table>' . "\n";
        } //elseif (has_capability('mod/iassign:submitiassign', $this->context, $USER->id))
        else
            echo '</td></tr></table>' . "\n";
    }

    /**
     * Display activity current
     */
    function view_iassign_current() {

        global $USER, $CFG, $COURSE, $DB, $OUTPUT;
        $id = $this->cm->id;
        $iassignid = $this->iassign->id;

        //search data of current activity
        $iassign_statement = $this->activity->get_activity(); //search data of current activity
        $ilm = new ilm($iassign_statement->iassign_ilmid);
        $iassign = $DB->get_record("iassign", array("id" => $iassignid));

        //log record
        $info = $iassign->name . ":" . $iassign_statement->name;
        add_to_log($this->course->id, "iassign", "view", "view.php?id={$this->cm->id}", $info, $this->cm->id, $USER->id);

        echo $OUTPUT->header();

        //Search of iLM data used in the current activity
        $iassign_ilm = $DB->get_record("iassign_ilm", array("id" => $iassign_statement->iassign_ilmid));


        if ($this->action == 'viewsubmission') {
            if (!empty($this->iassign_submission_current) or $this->iassign_submission_current != 0)
                $iassign_submission = $DB->get_record("iassign_submission", array("id" => $this->iassign_submission_current)); // data about activity current
            else
                $iassign_submission = $DB->get_record("iassign_submission", array("iassign_statementid" => $this->activity->get_activity()->id, "userid" => $this->userid_iassign)); // data about student solution
        }
        else {
            $iassign_submission = $DB->get_record("iassign_submission", array("iassign_statementid" => $this->activity->get_activity()->id, "userid" => $this->userid_iassign)); // data about student solution
        }

        if ($iassign_submission)
            $this->update_comment($iassign_submission->id);

        $file = $iassign_statement->file;
        $this->bottonPost = 0; //hide submit button
        $this->write_solution = 0; //disable recording solution
        $this->view_iassign = false; //disable visualization of activity
        $repeat = "";
        $last_iassign = "";
        $answer = "";
        $comment = "";

        //leo
        //TODO Verificar se o correto eh '$this->context' ou '$USER->context' como deixei
        if (($this->action != 'viewsubmission') && has_capability('mod/iassign:evaluateiassign', $USER->context, $USER->id)) {

            //---> access teacher for test

            if ($iassign_statement->type_iassign != 1) //activity of type example - not submit button for submission
                $this->bottonPost = 1;

            echo $OUTPUT->box('<p><strong>' . get_string('area_specific_teacher', 'iassign') . '</strong></p>');
            $this->activity->view_dates();
            $USER->iassignEdit = $this->bottonPost;
            $this->activity->show_info_iassign();

            if ($iassign_submission)
                $param_aux = "action=get_answer&iassign_submission_current=" . $iassign_submission->id . "&id=" . $id . "&iassign_current=" . $this->activity->get_activity()->id . "&write_solution=" . $this->write_solution . "&userid_iassign=" . $USER->id;
            else
                $param_aux = "action=get_answer&id=" . $id . "&iassign_current=" . $this->activity->get_activity()->id . "&write_solution=" . $this->write_solution . "&userid_iassign=" . $USER->id;

            $enderecoPOST = "" . $CFG->wwwroot . "/mod/iassign/view.php?" . $param_aux;
            if ($ilm->confirms_jar($iassign_statement->file, $iassign_ilm->file_jar, $this->cm->id))
                echo $OUTPUT->box($ilm->view_iLM($iassign_statement, $answer, $enderecoPOST, true));
        } //if (($this->action != 'viewsubmission') && has_capability('mod/iassign:evaluateiassign', $USER->context, $USER->id))
        elseif (($this->action == 'viewsubmission') && has_capability('mod/iassign:evaluateiassign', $USER->context, $USER->id)) { //teacher can evaluate
            //----> area teacher evaluate
            $row = optional_param('row', 0, PARAM_INT);
            $column = optional_param('column', 0, PARAM_INT);

            $link_next = icons::insert('right_disable');
            $link_previous = icons::insert('left_disable');
            $link_up = icons::insert('up_disable');
            $link_down = icons::insert('down_disable');

            //next_activity
            if ($USER->matrix_iassign[$row][$column]->iassign_next != -1) {
                $url_next = "view.php?action=viewsubmission&id=$id&iassign_submission_current=" . $USER->matrix_iassign[$row][$column + 1]->iassign_submission_current . "&userid_iassign=$this->userid_iassign&iassign_current=" . $USER->matrix_iassign[$row][$column]->iassign_next . "&view_iassign=report&row=" . ($row) . "&column=" . ($column + 1);
                $link_next = "<a href='" . $url_next . "'>" . (icons::insert('next_activity')) . "</a>";
            }
            //previous_activity
            if ($USER->matrix_iassign[$row][$column]->iassign_previous != -1) {
                $url_previous = "view.php?action=viewsubmission&id=$id&iassign_submission_current=" . $USER->matrix_iassign[$row][$column - 1]->iassign_submission_current . "&userid_iassign=$this->userid_iassign&iassign_current=" . $USER->matrix_iassign[$row][$column]->iassign_previous . "&view_iassign=report&row=" . ($row) . "&column=" . ($column - 1);
                $link_previous = "<a href='" . $url_previous . "'>" . (icons::insert('previous_activity')) . "</a>";
            }
            //previous_student
            if ($USER->matrix_iassign[$row][$column]->user_next != -1) {
                $url_down = "view.php?action=viewsubmission&id=$id&iassign_submission_current=" . $USER->matrix_iassign[$row + 1][$column]->iassign_submission_current . "&userid_iassign=" . $USER->matrix_iassign[$row][$column]->user_next . "&iassign_current=" . $this->activity->get_activity()->id . "&view_iassign=report&row=" . ($row + 1) . "&column=" . ($column);
                $link_down = "<a href='" . $url_down . "'>" . (icons::insert('previous_student')) . "</a>";
            }
            //next_student
            if ($USER->matrix_iassign[$row][$column]->user_previous != -1) {
                $url_up = "view.php?action=viewsubmission&id=$id&iassign_submission_current=" . $USER->matrix_iassign[$row - 1][$column]->iassign_submission_current . "&userid_iassign=" . $USER->matrix_iassign[$row][$column]->user_previous . "&iassign_current=" . $this->activity->get_activity()->id . "&view_iassign=report&row=" . ($row - 1) . "&column=" . ($column);
                $link_up = "<a href='" . $url_up . "'>" . (icons::insert('next_student')) . "</a>";
            }
            if ($iassign_submission)
                $answer = $iassign_submission->answer;
            $last_iassign = get_string('last_iassign', 'iassign');

            $user_answer = $DB->get_record("user", array('id' => $this->userid_iassign));

            // Messages related to due date (and user role)
            $this->activity->view_dates();

            echo $OUTPUT->box_start();
            echo '<table width=100% border=0 valign="top"><tr>' . "\n";
            echo '<td width=80%><font color="blue"><strong>' . get_string('area_available', 'iassign') . '</strong></font><br>' . "\n";
            echo $OUTPUT->user_picture($user_answer);
            echo '&nbsp;' . $user_answer->firstname . '&nbsp;' . $user_answer->lastname;
            echo '</td>' . "\n";
            echo '<td width=20% align=right>' . "\n";
            echo '<table width=50 cellpadding="0">';
            echo '<tr><td colspan=2 align=center>' . $link_up . '</td></tr>' . "\n";
            echo '<tr><td align=center>' . $link_previous . '</td>' . "\n";
            echo '<td align=center>' . $link_next . '</td></tr>' . "\n";
            echo '<td colspan=2 align=center>' . $link_down . '</td></tr>' . "\n";
            echo '</table>' . "\n";
            echo '</td></tr></table>' . "\n";
            echo $OUTPUT->box_end();

            echo $OUTPUT->box_start();
            echo '<table width=100% border=0 valign="top"><tr>' . "\n";
            echo '<td width=60% valign="top">' . "\n";
            echo '<strong>' . get_string('proposition', 'iassign') . '</strong>' . "\n";
            echo '<p>' . $iassign_statement->proposition . '</p>' . "\n";

            if ($iassign_statement->automatic_evaluate == 1)
                $resp = get_string('yes');
            else
                $resp = get_string('no');
            echo '<p>' . get_string('automatic_evaluate', 'iassign') . '&nbsp;' . $resp . '</p>' . "\n";
            if ($iassign_statement->show_answer == 1)
                $resp = get_string('yes');
            else
                $resp = get_string('no');
            echo '<p>' . get_string('show_answer', 'iassign') . '&nbsp;' . $resp . '</p>' . "\n";
            echo '</td>';

            if ($iassign_statement->type_iassign == 3) {

                echo '<td width=40% valign="top" align="left">';
                echo '<strong>' . get_string('status', 'iassign') . '</strong>' . "\n";

                //check status of solution sent by the student
                if ($iassign_submission) {
                    switch ($iassign_submission->status) {
                        case 3:
                            echo icons::insert('correct') . '&nbsp;' . get_string('correct', 'iassign') . '&nbsp;' . $comment;
                            break;
                        case 2:
                            echo icons::insert('incorrect') . '&nbsp;' . get_string('incorrect', 'iassign') . '&nbsp;' . $comment;
                            break;
                        case 1:
                            echo icons::insert('post') . '&nbsp;' . get_string('post', 'iassign') . '&nbsp;' . $comment;
                            break;
                        default:
                            echo icons::insert('not_post') . '&nbsp;' . get_string('not_post', 'iassign') . '&nbsp;' . $comment;
                            $last_iassign = get_string('no_MA_POST_Archive', 'iassign');
                    } // switch ($iassign_submission->status)
                } else {
                    echo icons::insert('not_post') . '&nbsp;' . get_string('not_post', 'iassign') . '&nbsp;' . $comment;
                    $last_iassign = get_string('no_MA_POST_Archive', 'iassign');
                }

                //update_status
                if ($iassign_submission && $iassign_submission->experiment > 0) {
                    $edit_status = $CFG->wwwroot . "/mod/iassign/view.php?action=edit_status&id=" . $id . "&userid_iassign=" . $this->userid_iassign . "&iassign_current=" . $this->activity->get_activity()->id . "&iassign_submission_current=" . $this->iassign_submission_current . "&row=" . ($row) . "&column=" . ($column);

                    echo "
           <script type='text/javascript'>
           //<![CDATA[
           function overwriteStatus (newstatus) {
             if (confirm('" . get_string('confirm_change_situation', 'iassign') . "')) {
                document.formEditStatus.return_status.value=newstatus;
                document.formEditStatus.submit();
                }
             else
                document.formEditStatus.return_status.value=-1;
             }
           //]]>
           </script>";

                    echo "<form name='formEditStatus' method='post' action='$edit_status' enctype='multipart/form-data'>\n";
                    echo ' <font color="blue"><strong>' . get_string('changeto', 'iassign') . "</strong></font>\n";
                    echo " <select name='status' onchange= 'overwriteStatus(this.value)'>\n" .
                    " <option value=\"3\">" . get_string('correct', 'iassign') . "</option>\n" .
                    " <option value=\"2\">" . get_string('incorrect', 'iassign') . "</option>\n" .
                    " <option value=\"1\">" . get_string('post', 'iassign') . "</option>\n" .
                    " <option value=\"0\">" . get_string('not_post', 'iassign') . "</option>\n" .
                    " <option value=\"-1\" selected>" . get_string('newsituation', 'iassign') . "</option>\n" .
                    " </select>\n";
                    echo " <input type='hidden' name='return_status'>\n";
                    echo "</form>\n";

                    echo '<p><strong>' . get_string('grade_student', 'iassign') . '</strong>&nbsp;' . $iassign_submission->grade . "</p>\n";
                    echo '<p><strong>' . get_string('grade_iassign', 'iassign') . '</strong>&nbsp;' . $iassign_statement->grade . "</p>\n";
                    $edit_grade = $CFG->wwwroot . "/mod/iassign/view.php?action=edit_grade&id=" . $id . "&userid_iassign=" . $this->userid_iassign . "&iassign_current=" . $this->activity->get_activity()->id . "&iassign_submission_current=" . $this->iassign_submission_current . "&row=" . ($row) . "&column=" . ($column);
                    echo "
           <script type='text/javascript'>
           //<![CDATA[
           function overwriteGrade (newgrade,maxgrade) {
            if (newgrade<0 || newgrade>maxgrade) {
              alert('" . get_string('erro_grade', 'iassign') . " '+maxgrade)
              document.formEditGrade.return_grade.value=-1;
              document.formEditGrade.submit();
            }
            else {
              document.formEditGrade.return_grade.value=newgrade;
              document.formEditGrade.submit();
            }
           }
           //]]>
           </script>";
                    echo "<form name='formEditGrade' method='post' action='$edit_grade' enctype='multipart/form-data'>\n";
                    echo ' <font color="blue"><strong>' . get_string('changeto', 'iassign') . "</strong></font>" . "\n";
                    echo " <input type='text' name='grade' size='6'>";
                    echo " <input type='hidden' name='return_grade'> ";
                    echo " <input type=button value='" . get_string('confirm', 'iassign') . "' onClick = 'overwriteGrade(grade.value," . $iassign_statement->grade . ")' " .
                    "  title='" . get_string('confirm_new_grade', 'iassign') . "'>\n";
                    echo "</form>";

                    echo '<p><strong>' . get_string('experiment', 'iassign') . '</strong>&nbsp;' . $iassign_submission->experiment . '</p>';
                    echo '<p><strong>' . get_string('timemodified', 'iassign') . '</strong>&nbsp;' . userdate($iassign_submission->timemodified) . '</p>';
                    $teacher = $DB->get_record("user", array('id' => $iassign_submission->teacher));
                    if ($teacher)
                        echo '<p><strong>' . get_string('last_modification', 'iassign') . '</strong>&nbsp;' . $teacher->firstname . '</p>' . "\n";
                } // if ($iassign_submission->experiment > 0)
                echo '</td>';
            } //if ($iassign_statement->type_iassign == 3)

            echo '</tr></table>';
            echo $OUTPUT->box_end();
            $USER->iassignEdit = $this->bottonPost;

            if ($iassign_submission && $iassign_submission->experiment > 0) {

                echo $OUTPUT->box_start();
                echo '<p><strong>' . $last_iassign . '</strong></p>';
                if ($ilm->confirms_jar($iassign_statement->file, $iassign_ilm->file_jar, $this->cm->id)) {
                    $enderecoPOST = "";
                    echo $ilm->view_iLM($iassign_statement, $answer, $enderecoPOST, false);
                } //if ($this->confirms_jar($iassign_statement->file, $iassign_ilm->file_jar))
                echo $OUTPUT->box_end();
            } else {
                echo $OUTPUT->box('<p><strong>' . $last_iassign . '</strong></p>');
            }

            if ($iassign_statement->type_iassign == 3) {
                $output = '';
                $history_comment = '';
                if ($iassign_submission) {
                    $enderecoPOSTcomment = "" . $CFG->wwwroot . "/mod/iassign/view.php?id=" . $id . "&action=newcomment&iassign_current=" . $this->activity->get_activity()->id . "&iassign_submission_current=" . $iassign_submission->id . "&userid_iassign=" . $this->userid_iassign . "&row=" . ($row) . "&column=" . ($column);
                    $history_comment = $this->search_comment_submission($iassign_submission->id);
                } else {
                    $enderecoPOSTcomment = "" . $CFG->wwwroot . "/mod/iassign/view.php?id=" . $id . "&action=newcomment&iassign_current=" . $this->activity->get_activity()->id . "&userid_iassign=" . $this->userid_iassign . "&row=" . ($row) . "&column=" . ($column);
                }

                $output .= $OUTPUT->box_start();

                $output .= "<center><form name='formEnvioComment' id='formEnvioComment' method='post' action='$enderecoPOSTcomment' enctype='multipart/form-data'>\n<p>\n";
                $output .= "<textarea rows='2' cols='60' name='submission_comment'></textarea>";
                $output .= "</p><p><input type=submit value='" . get_string('submit_comment', 'iassign') . "'\></p>\n" .
                        $output .= "</form></center>\n";
                if (!empty($history_comment)) {
                    $output .= "  <table id='outlinetable' class='generaltable boxaligncenter' cellpadding='5' width='100%'> \n" .
                            "     <tr><th>" . get_string('history_comments', 'iassign') . "</th></tr>"; //
                    $output .= $history_comment;
                    $output .="</table>";
                }
                $output .= $OUTPUT->box_end();

                echo $output;
            } //if ($iassign_statement->type_iassign == 3)
        } // elseif (($this->action == 'viewsubmission')
        elseif (has_capability('mod/iassign:submitiassign', $USER->context, $USER->id)) { //access student
            //---> access student
            if ($iassign_statement->type_iassign == 1) {
                //// activity of type example - not submit button for submission
                $this->view_iassign = true;
            } elseif ($iassign_statement->type_iassign == 2 and $iassign_ilm->evaluate == 1) {
                //// activity of type test - iLM automatic evaluator - submit button for submission
                if ($iassign_statement->timeavailable < time() and $iassign_statement->timedue > time()) { //activity within of deadline
                    $this->bottonPost = 1;
                    $this->view_iassign = true;
                } else
                    $this->view_iassign = false;
            } elseif ($iassign_statement->type_iassign == 3) {

                ////atividade do tipo exercicio
                $this->view_iassign = true;
                if ($iassign_statement->timeavailable > time())
                    $this->view_iassign = false;
                elseif ($iassign_statement->timedue > time() or $iassign_statement->preventlate == 1) {// activity within of deadline
                    $this->bottonPost = 1; // mudei aqui para deixar o botão de envio
                    if ((!$iassign_submission) or $this->action == 'repeat' or ($iassign_submission and $iassign_submission->experiment < 1)) {
                        $this->bottonPost = 1;
                        $this->write_solution = 1;
                    } else {
                        $last_iassign = get_string('last_iassign', 'iassign');
                        if ($iassign_submission) {
                            $repeat = "<a href='view.php?action=repeat&id=" . $id . "&userid_iassign=$USER->id&iassign_current=" . $this->activity->get_activity()->id . "&iassign_submission_current=" . $iassign_submission->id . "'>" . icons::insert('repeat') . '&nbsp;' . get_string('repeat', 'iassign') . "</a>";
                            $answer = $iassign_submission->answer;
                        } else {
                            $repeat = "<a href='view.php?action=repeat&id=" . $id . "&userid_iassign=$USER->id&iassign_current=" . $this->activity->get_activity()->id . "'>" . icons::insert('repeat') . '&nbsp;' . get_string('repeat', 'iassign') . "</a>";
                        }
                    }
                } elseif ($iassign_statement->test == 1) { // allowed to test after of deadline
                    if ($this->action == 'repeat' or ($iassign_submission and $iassign_submission->experiment < 1)) {
                        $this->bottonPost = 1;
                        $this->write_solution = 0;
                    } else {
                        $last_iassign = get_string('last_iassign', 'iassign');
                        if ($iassign_submission) {
                            $repeat = "<a href='view.php?action=repeat&id=" . $id . "&userid_iassign=$USER->id&iassign_current=" . $this->activity->get_activity()->id . "&iassign_submission_current=" . $iassign_submission->id . "'>" . icons::insert('repeat') . '&nbsp;' . get_string('repeat', 'iassign') . "</a>";
                            $answer = $iassign_submission->answer;
                        } else {
                            $repeat = "<a href='view.php?action=repeat&id=" . $id . "&userid_iassign=$USER->id&iassign_current=" . $this->activity->get_activity()->id . "'>" . icons::insert('repeat') . '&nbsp;' . get_string('repeat', 'iassign') . "</a>";
                        }
                    }
                } //elseif ($iassign_statement->test == 1)
                elseif ($iassign_statement->test == 0)
                    $this->view_iassign = false;
            } //elseif ($iassign_statement->type_iassign == 3)

            if ($iassign_submission)
                $param_aux = "action=get_answer&iassign_submission_current=" . $iassign_submission->id . "&id=" . $id . "&iassign_current=" . $this->activity->get_activity()->id . "&write_solution=" . $this->write_solution . "&userid_iassign=" . $USER->id;
            else
                $param_aux = "action=get_answer&id=" . $id . "&iassign_current=" . $this->activity->get_activity()->id . "&write_solution=" . $this->write_solution . "&userid_iassign=" . $USER->id;
            $enderecoPOST = "" . $CFG->wwwroot . "/mod/iassign/view.php?" . $param_aux;

            $this->view_legend_icons();
            $this->activity->view_dates();

            if ($this->view_iassign) {

                echo '<table width=100% border=0 valign="top">' . "\n";
                echo '<tr><td width=60% valign="top">' . "\n";
                echo '<strong>' . get_string('proposition', 'iassign') . '</strong>' . "\n";
                echo '<p>' . $iassign_statement->proposition . '</p>' . "\n";
                $flag_dependency = true;

                if ($iassign_statement->type_iassign == 3) {
                    if ($iassign_statement->dependency == 0) {
                        echo '<strong>' . get_string('independent_activity', 'iassign') . '</strong>' . "\n";
                    } else {
                        $dependencys = explode(';', $iassign_statement->dependency);
                        echo '<p><strong>' . get_string('dependency', 'iassign') . '</strong></p>' . "\n";

                        foreach ($dependencys as $dependency) {
                            if ($dependency) {
                                $dependencyiassign = $DB->get_record("iassign_statement", array("id" => $dependency));
                                $dependencysubmissions = $DB->get_record("iassign_submission", array("iassign_statementid" => $dependencyiassign->id, 'userid' => $USER->id));
                                if ($dependencysubmissions) {
                                    if ($dependencysubmissions->status == 3)
                                        $icon = icons::insert('correct');
                                    elseif ($dependencysubmissions->status == 2) {
                                        $icon = icons::insert('incorrect');
                                        $flag_dependency = false;
                                    } elseif ($dependencysubmissions->status == 1) {
                                        $icon = icons::insert('post');
                                        $flag_dependency = false;
                                    } elseif ($dependencysubmissions->status == 0) {
                                        $icon = icons::insert('not_post');
                                        $flag_dependency = false;
                                    }
                                } else {
                                    $icon = icons::insert('not_post');
                                    $flag_dependency = false;
                                } //if ($dependencysubmissions)

                                echo '<p>&nbsp;' . $icon . $dependencyiassign->name . '</p>' . "\n";
                            } // if ($dependency)
                        } //foreach ($dependencys as $dependency)
                    } //if ($iassign_statement->dependency == 0)
                } // if ($iassign_statement->type_iassign == 3)

                if ($flag_dependency == false) {
                    echo '<strong>' . get_string('message_dependency', 'iassign') . '</strong>' . "\n";
                    $this->view_iassign = false;
                    echo '</tr></table>' . "\n";
                } else {
                    $this->view_iassign = true;
                    echo '</td>' . "\n";
                } //if ($flag_dependency == false)

                if ($this->view_iassign) {
                    if ($iassign_statement->type_iassign == 3) {// activity is present only if exercise
                        //receiver=1 - message to teacher
                        //receiver=2 - message to student
                        if ($iassign_submission) {
                            $verify_message = $DB->get_record('iassign_submission_comment', array('iassign_submissionid' => $iassign_submission->id, 'return_status' => 0, 'receiver' => 2));
                            if ($verify_message)
                                $comment = icons::insert('comment_unread');
                        }
                        echo '<td width=40% valign="top" align="left">';
                        echo '<strong>' . get_string('status', 'iassign') . '</strong>' . "\n";

                        if ($iassign_statement->show_answer == 1) {

                            //check status of solution sent by the student
                            if ($iassign_submission) {
                                switch ($iassign_submission->status) {
                                    case 3:
                                        echo icons::insert('correct') . '&nbsp;' . get_string('correct', 'iassign') . '&nbsp;' . $comment;
                                        break;
                                    case 2:
                                        echo icons::insert('incorrect') . '&nbsp;' . get_string('incorrect', 'iassign') . '&nbsp;' . $comment;
                                        break;
                                    case 1:
                                        echo icons::insert('post') . '&nbsp;' . get_string('post', 'iassign') . '&nbsp;' . $comment;
                                        break;
                                    default:
                                        echo icons::insert('not_post') . '&nbsp;' . get_string('not_post', 'iassign') . '&nbsp;' . $comment;
                                        $repeat = "";
                                        $last_iassign = "";
                                } //switch ($iassign_submission->status)
                            } else {
                                echo icons::insert('not_post') . '&nbsp;' . get_string('not_post', 'iassign') . '&nbsp;' . $comment;
                                $repeat = "";
                                $last_iassign = "";
                            }

                            if ($iassign_submission and $iassign_submission->experiment > 0) {
                                echo '<p><strong>' . get_string('grade_student', 'iassign') . ':</strong>&nbsp;' . $iassign_submission->grade;
                                echo '&nbsp;&nbsp;(' . get_string('grade_iassign', 'iassign') . ':&nbsp;' . $iassign_statement->grade . ')</p>' . "\n";

                                echo '<p><strong>' . get_string('experiment_student', 'iassign') . '</strong>&nbsp;' . $iassign_submission->experiment;

                                if ($iassign_statement->max_experiment == 0)
                                    echo '&nbsp;&nbsp(' . get_string('experiment_iassign', 'iassign') . '&nbsp;' . get_string('ilimit', 'iassign') . ')</p>' . "\n";
                                else {
                                    echo '&nbsp;&nbsp(' . get_string('experiment_iassign', 'iassign') . '&nbsp;' . $iassign_statement->max_experiment . ')</p>' . "\n";
                                    if ($iassign_submission->experiment >= $iassign_statement->max_experiment) {
                                        $repeat = "";
                                        $last_iassign.="&nbsp;<font color=red>" . get_string('attempts_exhausted', 'iassign') . '</font>' . "\n";
                                        $this->bottonPost = 0;
                                        $this->write_solution = 0;
                                    } // if ($iassign_submission->experiment >= $iassign_statement->max_experiment)
                                } //if ($iassign_statement->max_experiment == 0)

                                echo '<p><strong>' . get_string('timemodified', 'iassign') . '</strong>&nbsp;' . userdate($iassign_submission->timemodified) . '</p>' . "\n";
                                $teacher = $DB->get_record("user", array('id' => $iassign_submission->teacher));
                                if ($teacher)
                                    echo '<p><strong>' . get_string('last_modification', 'iassign') . '</strong>&nbsp;' . $teacher->firstname . '</p>' . "\n"; //"
                            } // if ($iassign_submission->experiment > 0)
                        } // if ($iassign_statement->show_answer==1)
                        else {
                            if ($iassign_submission->status == 0) {
                                echo icons::insert('not_post') . '&nbsp;' . get_string('not_post', 'iassign') . '&nbsp;' . $comment;
                                $repeat = "";
                                $last_iassign = "";
                            } elseif ($iassign_submission->status == 1) {
                                echo icons::insert('post') . '&nbsp;' . get_string('post', 'iassign') . '&nbsp;' . $comment;
                            }
                        }

                        echo '</td>';
                    } //if ($iassign_statement->type_iassign == 3)

                    echo '</tr></table>' . "\n";

                    //show iLM
                    echo $OUTPUT->box_start();
                    echo '<table width=100% border=0 valign="top">' . "\n";
                    echo '<td width=80% align="left">';
                    echo '<strong>' . $last_iassign . '</strong></td>' . "\n";
                    echo '<td width=20% align="rigth">';
                    echo $repeat;
                    echo '</td></tr></table>' . "\n";

                    $output = '';
                    // $output .= "<center><form name='formEnvio' id='formEnvio' method='post' action='$enderecoPOST' enctype='multipart/form-data'>";
                    // $enderecoPOSTcomment = "" . $CFG->wwwroot . "/mod/iassign/view.php?id=" . $id . "&action=newcomment&iassign_current=" . $this->activity->get_activity()->id  . "&iassign_submission_current=" . $iassign_submission->id . "&userid_iassign=" . $this->userid_iassign;

                    if ($ilm->confirms_jar($iassign_statement->file, $iassign_ilm->file_jar, $this->cm->id)) {
                        $USER->iassignEdit = $this->bottonPost;
                        if (!$iassign_submission or $this->action == 'repeat') {//or $iassign_submission->answer==0
                            $output .= $ilm->view_iLM($iassign_statement, $answer, $enderecoPOST, true);
                        } elseif ($iassign_submission and $iassign_submission->answer == '0') {
                            $answer = "";
                            $output .= $ilm->view_iLM($iassign_statement, $answer, $enderecoPOST, true);
                        }
                        else
                            $output .= $ilm->view_iLM($iassign_statement, $answer, $enderecoPOST, false);
                    } // if ($ilm->confirms_jar($iassign_statement->file, $iassign_ilm->file_jar))
                    echo $OUTPUT->box_end();


                    if ($iassign_statement->type_iassign == 3) {


                        $history_comment = '';
                        if ($iassign_submission) {
                            $history_comment = $this->search_comment_submission($iassign_submission->id);
                        }

                        if (!empty($history_comment)) {
                            $output .= "
                      <table id='outlinetable' class='generaltable boxaligncenter' cellpadding='5' width='100%'>
                      <tr><th>" . get_string('history_comments', 'iassign') . "</th></tr>"; //
                            $output .= $history_comment;
                            $output .="</table>";
                        }
                        $output .= "</form></center>";
                        echo $output;
                    } //if ($iassign_statement->type_iassign == 3)
                } //if ($this->view_iassign)
            } //if ($this->view_iassign)
        }  //elseif (has_capability('mod/iassign:submitiassign', $this->context, $USER->id))
        // final block 'studant'
        echo $OUTPUT->footer();
        die;
    }

    //function view_iassign_current ()

    /**
     * Display report of performance
     */
    function report() {
        global $USER, $CFG, $DB, $OUTPUT;
        $id = $this->cm->id;
        $iassign_list = $DB->get_records_list('iassign_statement', 'iassignid', array('iassignid' => $this->iassign->id), "position ASC");


        if ($this->action != 'print') {
            $title = get_string('report', 'iassign');
            echo $OUTPUT->header();
        } //if ($this->action != 'print')
        echo $OUTPUT->box_start();

        $this->view_legend_icons();
        echo '<p>' . get_string('ps_experiment', 'iassign') . '</p>';
        echo '<p>' . get_string('ps_comment', 'iassign') . '</p>';
        echo $OUTPUT->box_end();

        if ($this->action == 'print')
            echo '<table border=1 width="100%">' . "\n";
        else
            echo '<table id="outlinetable" class="generaltable boxaligncenter" cellpadding="5" width="100%">' . "\n";
        echo '<tr><th colspan=2 class="header c1">' . $this->iassign->name . '</th></tr>' . "\n";
        // $num = array();
        $i = 1;
        $num = array();


        foreach ($iassign_list as $iassign) {
            $test_exercise = "";
            $iassign_submission = $DB->get_records("iassign_submission", array("iassign_statementid" => $iassign->id));
            if (($iassign_submission) && $iassign->type_iassign < 3) {
                $test_exercise = "&nbsp;<b>(" . get_string('iassign_exercise', 'iassign') . ")</b>";
            }// if (($iassign_submission) && $iassign->type_iassign < 3)
            if ($iassign->type_iassign == 3 || ($iassign_submission)) {
                $num[$i] = new object();
                $num[$i]->name = $iassign->name;
                $num[$i]->id = $i;
                $num[$i]->iassignid = $iassign->id;
                echo ' <tr >' . "\n";
                echo "  <td class=\"cell c1 numviews\" width=5% align='center'><strong>" . $num[$i]->id . "</strong></td>\n";
                echo "<td class=\"cell c0 actvity\">";
                echo '&nbsp;' . $num[$i]->name . '&nbsp;' . $test_exercise . "</td>";
                echo ' </tr>' . "\n";
                $i++;
            } // if ($iassign->type_iassign == 3 || ($iassign_submission))
        } // foreach ($iassign_list as $iassign)
        echo "</table>";
        echo "<p></p>";
        if ($this->action == 'print')
            echo '<table border=1 width="100%">' . "\n";
        else
            echo '<table id="outlinetable" class="generaltable boxaligncenter" cellpadding="5" width="100%">' . "\n";
        $context = get_context_instance(CONTEXT_COURSE, $this->course->id);
        if ($i > 1) {

            /*
              $role = $DB->get_record_sql("SELECT s.id, s.shortname
              FROM {$CFG->prefix}role s
              WHERE s.shortname = 'student'");
             */
            $params = array('shortname' => 'student');
            $role = $DB->get_record_sql('SELECT s.id, s.shortname
              FROM {role} s
              WHERE s.shortname = :shortname', $params); // " - jed/emacs


            /*
              $students = $DB->get_records_sql("SELECT s.userid, a.firstname, a.lastname
              FROM {$CFG->prefix}role_assignments s, {$CFG->prefix}user a
              WHERE s.contextid = '$context->id'
              AND s.userid = a.id
              AND s.roleid = '$role->id'
              ORDER BY `a`.`firstname` ASC,`a`.`lastname` ASC");
             */
            $params = array('contextid' => $context->id, 'roleid' => $role->id);
            $students = $DB->get_records_sql('SELECT s.userid, a.firstname, a.lastname
                             FROM {role_assignments} s, {user} a
                             WHERE s.contextid = :contextid
                             AND s.userid = a.id
                             AND s.roleid = :roleid
                             ORDER BY `a`.`firstname` ASC,`a`.`lastname` ASC', $params); // " - jed/emacs


            echo '<tr><th class="header c1">' . get_string('students', 'iassign') . '</th>' . "\n";
            for ($j = 1; $j < $i; $j++) {
                $sum_iassign_correct[$j] = 0;
                echo '<th class="header c1" scope="col">' . $num[$j]->id . '</th>' . "\n"; // <th class="header c1" scope="col">
            }

            $sum_iassign = $j - 1;
            echo '</tr>' . "\n";
            $total = 0;
            $sum_student = 0;
            $comment = icons::insert('comment_read');
            $sum_comment = 0;
            $sum_correct_iassign = array();
            $sum_correct_student = array();

            $USER->matrix_iassign = array();
            if ($students) {
                $w = 0;
                foreach ($students as $tmp) {
                    $users_array[$w] = $tmp;
                    $w++;
                }

                for ($x = 0; $x < $w; $x++) {

                    echo '<tr>' . "\n";
                    $sum_student++;
                    $name = $users_array[$x]->firstname . '&nbsp;' . $users_array[$x]->lastname;
                    echo '  <td >' . $name . '</td>' . "\n";
                    $total_student = 0;

                    for ($j = 1; $j < $i; $j++) {
                        $sum_comment = 0;

                        $student_submissions = $DB->get_record("iassign_submission", array('iassign_statementid' => $num[$j]->iassignid, 'userid' => $users_array[$x]->userid)); // data about student solution

                        echo '  <td valign="bottom" align="center">' . "\n";
                        if ($student_submissions) {
                            $last_solution_submission = " title=\"" . userdate($student_submissions->timemodified) . "\" "; // timemodified: time of the last student solution
                            $tentativas = $student_submissions->experiment;



                            /*
                              $student_submissions_comment = $DB->get_record_sql("SELECT COUNT(iassign_submissionid)
                              FROM {$CFG->prefix}ia_assign_submissions_comment
                              WHERE iassign_submissionid = '$student_submissions->id'");
                             */
                            $params = array('iassign_submissionid' => $student_submissions->id);
                            $student_submissions_comment = $DB->get_record_sql('SELECT COUNT(iassign_submissionid)
                           FROM {iassign_submission_comment}
                           WHERE iassign_submissionid = :iassign_submissionid', $params); // " - jed/emacs


                            if ($student_submissions_comment)
                                foreach ($student_submissions_comment as $tmp)
                                    $sum_comment = $tmp;

                            //informations to nagevacao between activities
                            //previous
                            if ($j - 1 < 1 or $j == $i)
                                $iassign_previous = "-1";
                            else
                                $iassign_previous = $num[$j - 1]->iassignid;

                            if ($x - 1 < 0 or $x == $w)
                                $user_previous = "-1";
                            else
                                $user_previous = $users_array[$x - 1]->userid;

                            //next
                            if ($i - 1 > $j)
                                $iassign_next = $num[$j + 1]->iassignid;
                            else
                                $iassign_next = "-1";

                            if ($w - 1 > $x)
                                $user_next = $users_array[$x + 1]->userid;
                            else
                                $user_next = "-1";

                            $position = "&row= $x&column=$j";

                            $url = "" . $CFG->wwwroot . "/mod/iassign/view.php?action=viewsubmission&id=" . $id . "&iassign_submission_current=" . $student_submissions->id . "&userid_iassign=" . $users_array[$x]->userid . "&iassign_current=" . $num[$j]->iassignid . "&view_iassign=" . $this->view_iassign;
                            $url .= $position;

                            //receiver=1 - message to teacher
                            //receiver=2 - message to student

                            /*
                              $verify_message = $DB->get_record_sql("SELECT COUNT(iassign_submissionid)
                              FROM {$CFG->prefix}ia_assign_submissions_comment
                              WHERE iassign_submissionid = '$student_submissions->id'
                              AND return_status='0'
                              AND receiver='1'");
                             */

                            $params = array('iassign_submissionid' => $student_submissions->id, 'return_status' => '0', 'receiver' => '1');
                            $verify_message = $DB->get_record_sql('SELECT COUNT(iassign_submissionid)
                             FROM {iassign_submission_comment}
                             WHERE iassign_submissionid = :iassign_submissionid
                             AND return_status= :return_status
                             AND receiver= :receiver', $params); // " - jed/emacs



                            if ($verify_message)
                                foreach ($verify_message as $tmp)
                                    $sum_verify_message = $tmp;

                            if ($sum_verify_message > 0)
                                $comment = icons::insert('comment_unread');
                            else
                                $comment = icons::insert('comment_read');

                            if ($student_submissions->status == 3) {
                                $sum_iassign_correct[$j]++;
                                $total_student++;
                                $feedback = icons::insert('correct');
                            } //if ($student_submissions->status == 3)
                            elseif ($student_submissions->status == 2) {
                                $feedback = icons::insert('incorrect');
                            } //elseif ($student_submissions->status == 2)
                            elseif ($student_submissions->status == 1) {
                                $feedback = icons::insert('post');
                            } //elseif ($student_submissions->status == 1)
                            elseif ($student_submissions->status == 0) {
                                $feedback = icons::insert('not_post');
                            } //elseif ($student_submissions->status == 0)

                            if ($this->action != 'print') {
                                echo '<table><tr>';
                                if ($tentativas > 0)
                                    echo '<td> <a href="' . $url . '" ' . $last_solution_submission . '>' . $feedback . '</a> &nbsp;(' . $tentativas . ')</td>' . chr(13);
                                else
                                    echo '<td> <a href="' . $url . '" ' . $last_solution_submission . '>' . $feedback . '</a> </td>' . chr(13);
                                echo '<td> &nbsp; </td>';
                                echo '</tr><tr>';
                                echo '<td> &nbsp;</td>';
                                if ($sum_comment > 0)
                                    echo '<td>  <a href="' . $url . '"> ' . $comment . '</a> &nbsp;(' . $sum_comment . ') </td>' . chr(13);
                                else
                                    echo '<td> &nbsp;</td>';
                                echo '</tr></table>';
                            }

                            if ($this->action == 'print')
                                echo $feedback . '&nbsp;(' . $tentativas . ')<br>' . $comment . '&nbsp;(' . $sum_comment . ')&nbsp;' . "\n";
                        } //if ($student_submissions)
                        else {

                            //informations to nagevacao between activities
                            //previous
                            if ($j - 1 < 1 or $j == $i)
                                $iassign_previous = "-1";
                            else
                                $iassign_previous = $num[$j - 1]->iassignid;

                            if ($x - 1 < 0 or $x == $w)
                                $user_previous = "-1";
                            else
                                $user_previous = $users_array[$x - 1]->userid;

                            //next
                            if ($i - 1 > $j)
                                $iassign_next = $num[$j + 1]->iassignid;
                            else
                                $iassign_next = "-1";

                            if ($w - 1 > $x)
                                $user_next = $users_array[$x + 1]->userid;
                            else
                                $user_next = "-1";

                            $position = "&row= $x&column=$j";

                            $url = "" . $CFG->wwwroot . "/mod/iassign/view.php?action=viewsubmission&id=" . $id . "&userid_iassign=" . $users_array[$x]->userid . "&iassign_current=" . $num[$j]->iassignid . "&view_iassign=" . $this->view_iassign;
                            $url .= $position;
                            $feedback = icons::insert('not_post');
                            if ($this->action == 'print')
                                echo $feedback . '&nbsp;(0)<br>' . $comment . '&nbsp;(' . $sum_comment . ')&nbsp;' . "\n";
                            else {
                                echo '<table><tr>';
                                echo '<td> <a href="' . $url . '">' . $feedback . '</a> </td>' . chr(13);
                                echo '<td> &nbsp; </td>';
                                echo '</tr><tr>' . chr(13);
                                echo '<td> &nbsp;</td>';
                                if ($sum_comment > 0)
                                    echo '<td> <a href="' . $url . '">' . $comment . '</a> &nbsp;(' . $sum_comment . ') </td>' . chr(13);
                                else
                                    echo '<td> &nbsp;</td>';
                                echo '</tr></table>';
                            }
                        } // else if ($student_submissions)
                        $USER->matrix_iassign[$x][$j] = new object();
                        $USER->matrix_iassign[$x][$j]->iassign_previous = $iassign_previous;
                        $USER->matrix_iassign[$x][$j]->user_previous = $user_previous;
                        $USER->matrix_iassign[$x][$j]->iassign_next = $iassign_next;
                        $USER->matrix_iassign[$x][$j]->user_next = $user_next;

                        if ($student_submissions)
                            $USER->matrix_iassign[$x][$j]->iassign_submission_current = $student_submissions->id;
                        else
                            $USER->matrix_iassign[$x][$j]->iassign_submission_current = 0;
                        echo '</td>' . "\n";
                    } // for ($j=1; $j<$i; $j++)

                    $total = $total + $total_student;
                    $porcentagem = ($total_student / ($j - 1)) * 100;
                    echo '</tr>' . "\n";
                    $sum_correct_student[$sum_student] = new object();
                    $sum_correct_student[$sum_student]->name = $name;
                    $sum_correct_student[$sum_student]->sum = $total_student;
                } // foreach ($students as $users_array[$x])
                //}
                for ($i = 1; $i < $j; $i++) {


                    if (is_null($sum_iassign_correct[$i]))
                        $sum_iassign_correct[$i] = 0;

                    $sum_correct_iassign[$i] = new object();
                    $sum_correct_iassign[$i]->sum = $sum_iassign_correct[$i];
                    $sum_correct_iassign[$i]->name = $num[$i]->name;
                } //for ($i = 1; $i < $j; $i++)
                // echo '</tr></table>' . "\n";
            }
        } // if ($i>1)
        else {
            print_string('no_activity', 'iassign');
        }// if ($i>1)
        echo "</table>";

        if ($this->action != 'print')
            echo $OUTPUT->footer();
        die;
    }

    /**
     * Display graphics of performance
     */
    function stats() {

        global $USER, $CFG, $DB, $OUTPUT;
        $id = $this->cm->id;
        $iassign_list = $DB->get_records_list('iassign_statement', 'iassignid', array('iassignid' => $this->iassign->id), "position ASC");

        if ($this->action != 'printstats')
            $title = get_string('graphic', 'iassign');

        $num = array();
        $sum_correct_iassign = array();
        $sum_correct_student = array();
        $sum_student = array();
        $i = 1;
        foreach ($iassign_list as $iassign) {
            $iassign_submission = $DB->get_records("iassign_submission", array("iassign_statementid" => $iassign->id));
            if ($iassign->type_iassign == 3) { //|| ($iassign_submission)
                $sum_iassign_correct[$i] = 0;

                $num[$i] = new object();
                $num[$i]->name = $iassign->name;
                $num[$i]->id = $i;
                $num[$i]->iassignid = $iassign->id;
                $i++;
            } //if ($iassign->type_iassign == 3)
        } // foreach ($iassign_list as $iassign)
        $sum_iassign = $i - 1;

        $context = get_context_instance(CONTEXT_COURSE, $this->course->id);

        if ($i > 1) {
            /*
              $role = $DB->get_record_sql("SELECT s.id, s.shortname
              FROM {$CFG->prefix}role s
              WHERE s.shortname = 'student'");
             */
            $params = array('shortname' => 'student');
            $role = $DB->get_record_sql('SELECT s.id, s.shortname
              FROM {role} s
              WHERE s.shortname = :shortname', $params); // " - jed/emacs

            /*
              $students = $DB->get_records_sql("SELECT s.userid, a.firstname, a.lastname
              FROM {$CFG->prefix}role_assignments s, {$CFG->prefix}user a
              WHERE s.contextid = '$context->id'
              AND s.userid = a.id
              AND s.roleid = '$role->id'
              ORDER BY `a`.`firstname` ASC,`a`.`lastname` ASC");
             */

            $params = array('contextid' => $context->id, 'roleid' => $role->id);
            $students = $DB->get_records_sql('SELECT s.userid, a.firstname, a.lastname
                             FROM {role_assignments} s, {user} a
                             WHERE s.contextid = :contextid
                             AND s.userid = a.id
                             AND s.roleid = :roleid
                             ORDER BY `a`.`firstname` ASC,`a`.`lastname` ASC', $params); // " - jed/emacs


            $total = 0;
            $sum_student = 0;
            $j = 0;
            $sum_correct_iassign = array();
            $sum_correct_student = array();
            $sum_experiment = array();
            if ($students) {
                foreach ($students as $users) {
                    $sum_student++;
                    $name = $users->firstname . '&nbsp;' . $users->lastname;
                    // rows
                    $total_student = 0;

                    for ($j = 1; $j < $i; $j++) {
                        $total_experiment = 0;
                        $student_submissions = $DB->get_record("iassign_submission", array('iassign_statementid' => $num[$j]->iassignid, 'userid' => $users->userid)); // data about student solution

                        if ($student_submissions) {
                            if ($student_submissions->status == 3) {
                                $sum_iassign_correct[$j]++;
                                $total_student++;
                            } // if ($student_submissions->status == 3)
                            $total_experiment+=$student_submissions->experiment;
                        }
                        $sum_experiment[$j] = $total_experiment;
                    } // for ($j=1; $j<$i; $j++)


                    $total = $total + $total_student;
                    $sum_correct_student[$sum_student] = new object();
                    $sum_correct_student[$sum_student]->name = $name;
                    $sum_correct_student[$sum_student]->sum = $total_student;
                } // foreach ($students as $users)
            }
            for ($i = 1; $i < $j; $i++) {
                if (is_null($sum_iassign_correct[$i]))
                    $sum_iassign_correct[$i] = 0;
                $sum_correct_iassign[$i] = new object();
                $sum_correct_iassign[$i]->sum = $sum_iassign_correct[$i];
                $sum_correct_iassign[$i]->name = $num[$i]->name;
                $sum_correct_iassign[$i]->experiment = $sum_experiment[$i];
            } //for ($i = 1; $i < $j; $i++)
        } // if ($i > 1)

        if ($this->action != 'printstats') {
            $title = get_string('graphic', 'iassign');
            echo $OUTPUT->header();
            $link_report = "<a href='" . $CFG->wwwroot . "/mod/iassign/view.php?id=" . $id . "&action=report&iassignid=" . $this->iassign->id . "'>" . icons::insert('view_report') . '&nbsp;' . get_string('report', 'iassign') . "</a>";
            $link_print_stats = "<a href='" . $CFG->wwwroot . "/mod/iassign/view.php?id=" . $id . "&action=printstats&&iassignid=" . $this->iassign->id . "'>" . icons::insert('print') . '&nbsp;' . get_string('print', 'iassign') . "</a>";
            echo '<table width=100%><tr>';
            echo'<td align="right">' . $link_print_stats . '</td>' . "\n";
            echo '<td width=15% align="right">' . $link_report . '</td>';
            echo '</td></tr></table>' . "\n";

            echo "<br><br>";
            echo '<table id="outlinetable" class="generaltable boxaligncenter" cellpadding="5" width="100%">' . "\n";
            echo '<tr><th colspan=5 class="header c1">' . get_string('distribution_activity', 'iassign') . '</th></tr>' . "\n";
            echo '<tr><td class=\'cell c0 actvity\' width=35%><strong>' . $this->iassign->name . '</strong></td>' . "\n";
            echo '<td class=\'cell c0 actvity\' width=35%><strong>' . get_string('percentage_correct', 'iassign') . '</strong></td>';
            echo '<td class=\'cell c0 actvity\' width=10% align="right"><strong>' . get_string('proportion_correct', 'iassign') . '</strong>';
            echo '<td class=\'cell c0 actvity\' width=10% align="right"><strong>' . get_string('sum_experiment', 'iassign') . '</strong></td>';
            echo '<td class=\'cell c0 actvity\' width=10% align="right"><strong>' . get_string('avg_experiment', 'iassign') . '</strong></td>';
            echo '</tr>' . "\n";
            $sum_correct = 0;
            if ($sum_correct_iassign) {
                foreach ($sum_correct_iassign as $iassign) {
                    if (is_null($iassign->experiment))
                        $iassign->experiment = 0;
                    $bar = "";
                    $sum = $iassign->sum;
                    $percent = ($sum / $sum_student) * 100;
                    $text = number_format($percent, 1) . '%';
                    $sum_correct+=$sum;
                    if ($sum > 0) {
                        for ($i = 1; $i < $percent * 2; $i++)
                            $bar.= icons::insert('hbar_blue');
                        $bar.=icons::insert('hbar_blue_r');
                    } // if ($sum > 0)
                    echo '<tr ><td class=\'cell c0 actvity\'width=35%>' . $iassign->name . '</td>' . "\n";
                    echo '<td class=\'cell c0 actvity\' width=35%>' . $bar . '&nbsp;' . $text . '</td>';
                    echo '<td class=\'cell c0 actvity\' width=10% align="right">' . $sum . '/' . $sum_student . '</td>';
                    echo '<td class=\'cell c0 actvity\' width=10% align="right">' . $iassign->experiment . '</td>';
                    echo '<td class=\'cell c0 actvity\' width=10% align="right">' . number_format($iassign->experiment / $sum_iassign, 1) . '</td>';
                    echo '</tr>' . "\n";
                } //foreach ($sum_correct_iassign as $iassign)
            }
            echo "</table>";
            echo "<br><br>";
            echo '<table id="outlinetable" class="generaltable boxaligncenter" cellpadding="5" width="100%">' . "\n";
            echo '<tr><th colspan=3 class="header c1">' . get_string('distribution_student', 'iassign') . '</th></tr>' . "\n";
            echo '<tr><td class=\'cell c0 actvity\' width=50%><strong>' . $this->iassign->name . '</strong></td>' . "\n";
            echo '<td class=\'cell c0 actvity\' width=40%><strong>' . get_string('percentage_correct', 'iassign') . '</strong></td>';
            echo '<td class=\'cell c0 actvity\' width=10% align="right"><strong>' . get_string('sum_correct', 'iassign') . '</strong></td>';
            echo '</tr>' . "\n";
            $sum_correct = 0;
            foreach ($sum_correct_student as $student) {
                $bar = "";
                $sum = $student->sum;
                $percent = ($sum / $sum_iassign) * 100;
                $text = number_format($percent, 1) . '%';
                $sum_correct+=$sum;
                if ($sum > 0) {
                    for ($i = 1; $i < $percent * 2; $i++)
                        $bar.= icons::insert('hbar_blue');
                    $bar.=icons::insert('hbar_blue_r');
                } // if ($sum > 0)
                echo '<tr ><td class=\'cell c0 actvity\'width=50%>' . $student->name . '</td>' . "\n";
                echo '<td class=\'cell c0 actvity\' width=40%>' . $bar . '&nbsp;' . $text . '</td>';
                echo '<td class=\'cell c0 actvity\' width=10% align="right">' . $sum . '/' . $sum_iassign . '</td>';
                echo '</tr>' . "\n";
            } //foreach ($sum_correct_student as $student)
            echo "</table>";
            echo "<br><br>";

            $var = 0;
            $cv = 0;
            $dv = 0;
            $avg = 0;
            if ($sum_correct_student) {
                $avg = $sum_correct / $sum_student;
                if ($avg > 0) {
                    foreach ($sum_correct_student as $student)
                        $var+=pow($student->sum - $avg, 2);

                    $var = $var / $sum_student;
                    $dv = sqrt($var);
                    $cv = ($dv / $avg) * 100;
                }
            }

            echo '<table id="outlinetable" class="generaltable boxaligncenter" cellpadding="5" width="100%">' . "\n";
            echo '<tr><th colspan=5 class="header c1">' . get_string('statistics', 'iassign') . '</th></tr>' . "\n";
            echo '<tr><td class=\'cell c0 actvity\' width=20% align="center"><strong>' . get_string('sum_activity', 'iassign') . '</strong></td>';
            echo '<td class=\'cell c0 actvity\' width=20% align="center"><strong>' . get_string('sum_student', 'iassign') . '</strong></td>';
            echo '<td class=\'cell c0 actvity\' width=20% align="center"><strong>' . get_string('mean_score', 'iassign') . '</strong></td>';
            echo '<td class=\'cell c0 actvity\' width=20% align="center"><strong>' . get_string('standard_deviation', 'iassign') . '</strong></td>';
            echo '<td class=\'cell c0 actvity\' width=20% align="center"><strong>' . get_string('coefficient_variation', 'iassign') . '</strong></td></tr>';
            echo '<tr><td class=\'cell c0 actvity\' width=20% align="center">' . $sum_iassign . '</td>';
            echo '<td class=\'cell c0 actvity\' width=20% align="center">' . $sum_student . '</td>';
            echo '<td class=\'cell c0 actvity\' width=20% align="center">' . number_format($avg, 1) . '</td>';
            echo '<td class=\'cell c0 actvity\' width=20% align="center">' . number_format($dv, 1) . '</td>';
            echo '<td class=\'cell c0 actvity\' width=20% align="center">' . number_format($cv, 1) . '%</td></tr>';
            echo "</table>";

            echo $OUTPUT->footer();
        } else {

            echo'<STYLE TYPE="text/css">
            <!--
             .boldtable
             {
             font-family:sans-serif;
             font-size:10pt;
             }
           -->
          </STYLE>'; //"

            echo '<table border=1 width=100%>' . "\n";
            echo '<tr><td colspan=3 align="center"><strong>' . get_string('distribution_activity', 'iassign') . '</strong></td></tr>' . "\n";
            echo '<tr><td width=50%><strong>' . $this->iassign->name . '</strong></td>' . "\n";
            echo '<td width=40%><strong>' . get_string('percentage_correct', 'iassign') . '</strong></td>';
            echo '<td width=10% align="right"><strong>' . get_string('sum_correct', 'iassign') . '</strong></td>';
            echo '</tr>' . "\n";
            $sum_correct = 0;
            foreach ($sum_correct_iassign as $iassign) {
                $bar = "";
                $sum = $iassign->sum;
                $percent = ($sum / $sum_student) * 100;
                $text = number_format($percent, 1) . '%';
                $sum_correct+=$sum;
                if ($sum > 0) {
                    for ($i = 1; $i < $percent * 2; $i++)
                        $bar.= icons::insert('hbar_blue');
                    $bar.=icons::insert('hbar_blue_r');
                } // if ($sum > 0)
                echo '<tr><td width=50%>' . $iassign->name . '</td>' . "\n";
                echo '<td width=40%>' . $bar . '&nbsp;' . $text . '</td>';
                echo '<td width=10% align="right">' . $sum . '/' . $sum_student . '</td>';
                echo '</tr>' . "\n";
            } //foreach ($sum_correct_iassign as $iassign)
            echo "</table>";
            echo "<br><br>";
            echo '<table border=1 class="boldtable" width=100%>' . "\n";
            echo '<tr><td colspan=3 align="center" ><strong>' . get_string('distribution_student', 'iassign') . '</strong></td></tr>' . "\n";
            echo '<tr><td width=50%><strong>' . $this->iassign->name . '</strong></td>' . "\n";
            echo '<td  width=40%><strong>' . get_string('percentage_correct', 'iassign') . '</strong></td>';
            echo '<td  width=10% align="right"><strong>' . get_string('sum_correct', 'iassign') . '</strong></td>';
            echo '</tr>' . "\n";
            $sum_correct = 0;
            foreach ($sum_correct_student as $student) {
                $bar = "";
                $sum = $student->sum;
                $percent = ($sum / $sum_iassign) * 100;
                $text = number_format($percent, 1) . '%';
                $sum_correct+=$sum;
                if ($sum > 0) {
                    for ($i = 1; $i < $percent * 2; $i++)
                        $bar.= icons::insert('hbar_blue');
                    $bar.=icons::insert('hbar_blue_r');
                } // if ($sum > 0)
                echo '<tr><td width=50%>' . $student->name . '</td>' . "\n";
                echo '<td width=40%>' . $bar . '&nbsp;' . $text . '</td>';
                echo '<td width=10% align="right">' . $sum . '/' . $sum_iassign . '</td>';
                echo '</tr>' . "\n";
            } //foreach ($sum_correct_student as $student)
            echo "</table>";
            echo "<br><br>";

            $var = 0;
            $cv = 0;
            $dv = 0;
            $avg = 0;
            if ($sum_correct_student) {
                $avg = $sum_correct / $sum_student;
                if ($avg > 0) {
                    foreach ($sum_correct_student as $student)
                        $var+=pow($student->sum - $avg, 2);
                    $var = $var / $sum_student;
                    $dv = sqrt($var);
                    $cv = ($dv / $avg) * 100;
                }
            }

            echo '<table border=1 class="boldtable" width=100%>' . "\n";
            echo '<tr><td colspan=5 align="center"><strong>' . get_string('statistics', 'iassign') . '</strong></th></tr>' . "\n";
            echo '<tr><td width=20% align="center"><strong>' . get_string('sum_activity', 'iassign') . '</strong></td>' . "\n";
            echo '<td  width=20% align="center"><strong>' . get_string('sum_student', 'iassign') . '</strong></td>' . "\n";
            echo '<td  width=20% align="center"><strong>' . get_string('mean_score', 'iassign') . '</strong></td>' . "\n";
            echo '<td  width=20% align="center"><strong>' . get_string('standard_deviation', 'iassign') . '</strong></td>' . "\n";
            echo '<td  width=20% align="center"><strong>' . get_string('coefficient_variation', 'iassign') . '</strong></td></tr>' . "\n";
            echo '<tr><td  width=20% align="center">' . $sum_iassign . '</td>' . "\n";
            echo '<td  width=20% align="center">' . $sum_student . '</td>' . "\n";
            echo '<td  width=20% align="center">' . number_format($avg, 1) . '</td>' . "\n";
            echo '<td  width=20% align="center">' . number_format($dv, 1) . '</td>' . "\n";
            echo '<td  width=20% align="center">' . number_format($cv, 1) . '%</td></tr> . "\n"';
            echo "</table>\n";
        } //if ($this->action != 'printstats')
        die;
    }

    /**
     * Display graphics of performance for students
     */
    function stats_students() {
        global $USER, $CFG, $DB, $OUTPUT;
        $id = $this->cm->id;
        $iassign_statement = $DB->get_records_sql("SELECT *
                             FROM {$CFG->prefix}iassign_statement s
                             WHERE s.iassignid = '{$this->iassign->id}'
                             AND s.type_iassign=3
                             ORDER BY `s`.`position`");

        $title = get_string('results', 'iassign');
        // echo $OUTPUT->header();

        $sum_correct = 0;
        $sum_incorrect = 0;
        $sum_post = 0;
        $sum_nopost = 0;
        $sum_iassign = count($iassign_statement);
        $bar_nopost = "";
        $bar_correct = "";
        $bar_incorrect = "";
        $bar_post = "";
        $text_nopost = "";
        $text_correct = "";
        $text_incorrect = "";

        foreach ($iassign_statement as $iassign) {
            $iassign_submission = $DB->get_record("iassign_submission", array('iassign_statementid' => $iassign->id, 'userid' => $USER->id)); // data about student solution
            if ($iassign_submission) {
                if ($iassign_submission->status == 3)
                    $sum_correct++;
                elseif ($iassign_submission->status == 2)
                    $sum_incorrect++;
                elseif ($iassign_submission->status == 1)
                    $sum_post++;
                elseif ($iassign_submission->status == 0 or (!$iassign_submission))
                    $sum_nopost++;
            } //if ($iassign_submission)
        } //foreach ($iassign_statement as $iassign)

        if ($sum_iassign > 0) {
            $percent_correct = ($sum_correct / $sum_iassign) * 100;
            $text_correct = number_format($percent_correct, 1) . '%';
        }

        if ($sum_correct > 0) {
            for ($i = 1; $i < $percent_correct * 2; $i++)
                $bar_correct.= icons::insert('hbar_green');
            $bar_correct.=icons::insert('hbar_green_r');
        } // if ($sum_correct > 0)

        if ($sum_iassign > 0) {
            $percent_incorrect = ($sum_incorrect / $sum_iassign) * 100;
            $text_incorrect = number_format($percent_incorrect, 1) . '%';
        }

        if ($sum_incorrect > 0) {
            for ($i = 1; $i < $percent_incorrect * 2; $i++)
                $bar_incorrect.= icons::insert('hbar_red');
            $bar_incorrect.=icons::insert('hbar_red_r');
        } // if ($sum_incorrect > 0)

        if ($sum_iassign > 0) {
            $percent_post = ($sum_post / $sum_iassign) * 100;
            $text_post = number_format($percent_post, 1) . '%';
        }
        if ($sum_post > 0) {
            for ($i = 1; $i < $percent_post * 2; $i++)
                $bar_post.= icons::insert('hbar_blue');
            $bar_post.=icons::insert('hbar_blue_r');
        } //if ($sum_post > 0)

        if ($sum_iassign > 0) {
            $percent_nopost = ($sum_nopost / $sum_iassign) * 100;
            $text_nopost = number_format($percent_nopost, 1) . '%';
        }
        if ($sum_nopost > 0) {
            for ($i = 1; $i < $percent_nopost * 2; $i++)
                $bar_nopost.= icons::insert('hbar_orange');
            $bar_nopost.=icons::insert('hbar_orange_r');
        } //if ($sum_nopost > 0)

        echo $OUTPUT->header();
        $link_return = "&nbsp;<a href='" . $this->return . "'>" . icons::insert('home') . get_string('activities_page', 'iassign') . "</a>";
        echo '<table width=100%><tr>';
        echo'<td align="right">' . $link_return . '</td>' . "\n";
        echo '</td></tr></table>' . "\n";

        echo "<br><br>";
        echo '<table id="outlinetable" class="generaltable boxaligncenter" cellpadding="5" width="100%">' . "\n";

        echo '<tr><th colspan=3 class="header c1">';
        // helpbutton('legend', get_string('legend', 'iassign'), 'iassign', $image = true, $linktext = false, $text = '', $return = false,
        // $imagetext = '');
        echo $this->iassign->name . '</th></tr>' . "\n";

        echo '<tr ><td class=\'cell c0 actvity\'width=50%>' . get_string('correct', 'iassign') . '</td>' . "\n";
        echo '<td class=\'cell c0 actvity\' width=40%>' . $bar_correct . '&nbsp;' . $text_correct . '</td>';
        echo '<td class=\'cell c0 actvity\' width=10% align="right">' . $sum_correct . '/' . $sum_iassign . '</td>';
        echo '</tr>' . "\n";
        echo '<tr ><td class=\'cell c0 actvity\'width=50%>' . get_string('incorrect', 'iassign') . '</td>' . "\n";
        echo '<td class=\'cell c0 actvity\' width=40%>' . $bar_incorrect . '&nbsp;' . $text_incorrect . '</td>';
        echo '<td class=\'cell c0 actvity\' width=10% align="right">' . $sum_incorrect . '/' . $sum_iassign . '</td>';
        echo '</tr>' . "\n";
        if ($sum_post) {
            echo '<tr ><td class=\'cell c0 actvity\'width=50%>' . get_string('post', 'iassign') . '</td>' . "\n";
            echo '<td class=\'cell c0 actvity\' width=40%>' . $bar_post . '&nbsp;' . $text_post . '</td>';
            echo '<td class=\'cell c0 actvity\' width=10% align="right">' . $sum_post . '/' . $sum_iassign . '</td>';
            echo '</tr>' . "\n";
        } // if ($sum_post)
        echo '<tr ><td class=\'cell c0 actvity\'width=50%>' . get_string('not_post', 'iassign') . '</td>' . "\n";
        echo '<td class=\'cell c0 actvity\' width=40%>' . $bar_nopost . '&nbsp;' . $text_nopost . '</td>';
        echo '<td class=\'cell c0 actvity\' width=10% align="right">' . $sum_nopost . '/' . $sum_iassign . '</td>';
        echo '</tr>' . "\n";
        echo "</table>";
        echo "<br><br>";
        echo '<table id="outlinetable" class="generaltable boxaligncenter" cellpadding="5" width="100%">' . "\n";
        echo '<tr><th colspan=3 class="header c1">' . get_string('grades', 'iassign') . '</th></tr>' . "\n";
        echo '<tr><td class=\'cell c0 actvity\' width=50%><strong>' . $this->iassign->name . '</strong></td>' . "\n";
        echo '<td class=\'cell c0 actvity\' width=25% align=right><strong>' . get_string('grade_student', 'iassign') . '</strong></td>' . "\n";
        echo '<td class=\'cell c0 actvity\' width=25% align=right><strong>' . get_string('grade_iassign', 'iassign') . '</strong></tr>' . "\n";

        $sum_grade = 0;
        $sum_grade_student = 0;
        $avg = 0;
        foreach ($iassign_statement as $iassign) {
            $iassign_submission = $DB->get_record("iassign_submission", array('iassign_statementid' => $iassign->id, 'userid' => $USER->id));
            if (!$iassign_submission) {
                $iassign_submission = new object();
                $iassign_submission->grade = 0;
            }
            echo '<tr ><td class=\'cell c0 actvity\'width=50%>' . $iassign->name . '</td>' . "\n";
            echo '<td class=\'cell c0 actvity\' width=25% align=right>' . $iassign_submission->grade . '</td>';
            echo '<td class=\'cell c0 actvity\' width=25% align=right>' . $iassign->grade . '</td>';
            echo '</tr>' . "\n";

            $sum_grade += $iassign->grade;
            $sum_grade_student += $iassign_submission->grade;
        } //foreach ($iassign_statement as $iassign)
        if ($sum_grade > 0)
            $avg = $sum_grade_student / $sum_grade * 100;

        echo '<tr><td class=\'cell c0 actvity\' width=50%><strong>' . get_string('total', 'iassign') . '</strong></td>' . "\n";
        echo '<td class=\'cell c0 actvity\' width=25% align=right><strong>' . $sum_grade_student . '</strong></td>' . "\n";
        echo '<td class=\'cell c0 actvity\' width=25% align=right><strong>' . $sum_grade . '</strong></tr>' . "\n";
        echo '<tr><td class=\'cell c0 actvity\' width=25% align=left><strong>' . get_string('percentage_correct', 'iassign') . '</strong></td>' . "\n";
        echo '<td colspan=2 class=\'cell c0 actvity\' align=right><strong>' . number_format($avg, 1) . '%</strong></tr>' . "\n";
        echo "</table>";

        echo $OUTPUT->footer();
        die;
    }

    /**
     * Display page of iAssigns
     */
    function view_iassigns() {
        global $USER, $CFG, $COURSE, $DB, $OUTPUT;
        $id = $this->cm->id;

        $iassign_list = $DB->get_records_list('iassign_statement', 'iassignid', array('iassignid' => $this->iassign->id), 'position ASC');

        echo $OUTPUT->box_start();

        echo '<table width=100% border=0><tr>' . "\n";
        $link_add = "<a href='" . $CFG->wwwroot . "/mod/iassign/view.php?id=" . $id . "&action=add&iassignid=" . $this->iassign->id . "'>" . icons::insert('add_iassign') . get_string('add_iassign', 'iassign') . "</a>";
        $link_report = "<a href='" . $CFG->wwwroot . "/mod/iassign/view.php?id=" . $id . "&action=report&iassignid=" . $this->iassign->id . "'>" . icons::insert('view_report') . get_string('report', 'iassign') . "</a>";

        // TODO: esta consulta esta sendo feita novamente na linha +/- 2258
        if (has_capability('mod/iassign:viewiassignall', $this->context, $USER->id)) { // could be "has_capability('mod/iassign:viewiassignall', $this->context, $USER->id)"
            // Has capability to see "report": teacher or up
            echo '<td width=10% align="left">' . "\n";
            echo $link_report;
            echo '</td>' . "\n";
        } //if (has_capability('mod/iassign:viewiassignall', $this->context, $USER->id))
        if (has_capability('mod/iassign:editiassign', $this->context, $USER->id)) {
            echo '<td width=15% align="left">' . "\n";
            echo $link_add;
            echo "</td>\n";
        } //if (has_capability('mod/iassign:editiassign', $this->context, $USER->id))
        if (has_capability('mod/iassign:editiassign', $this->context, $USER->id)) {
            if ($iassign_list) {
                echo '<td align="right">' . "\n";

                // $USER->iassignEdit == 0 view 'Turn editing off'
                // $USER->iassignEdit == 1 view 'Turn editing on'
                if (!isset($USER->iassignEdit))
                    $USER->iassignEdit = 0;

                if ($USER->iassignEdit == 0) {
                    $bottonEdit_message = get_string('turneditingon', 'iassign');
                    $botton = 1;
                }  // if ($USER->iassignEdit == 0)
                elseif ($USER->iassignEdit == 1) {
                    $bottonEdit_message = get_string('turneditingoff', 'iassign');
                    $botton = 0;
                } //elseif ($USER->iassignEdit == 1)
                $editPost = "" . $CFG->wwwroot . "/mod/iassign/view.php?id=" . $id . "&botton=" . $botton;
                echo "\n<form name='formEditPost' id='formEditPost' method='post' action='$editPost' enctype='multipart/form-data'>\n";
                echo " <input type=submit value='$bottonEdit_message'/>\n";
                echo "</form>\n";
                echo "</td>\n";
            } //if ($iassign_list)
        }
        echo '</tr></table>' . "\n";

        if (has_capability('mod/iassign:submitiassign', $this->context, $USER->id))
            $this->view_legend_icons();

        echo $OUTPUT->box_end();

        $iassign_array_exercise = array();
        $i_exercise = 0;
        $iassign_array_test = array();
        $i_test = 0;
        $iassign_array_example = array();
        $i_example = 0;
        $iassign_array_general = array();
        $i_general = 0;

        if ($iassign_list) {

            if ($this->iassign->activity_group == 0) {
                //if ($iassign_list) {
                foreach ($iassign_list as $iassign) {
                    $iassign_array_general[$i_general] = $iassign;
                    $i_general++;
                } //foreach ($iassign_list as $iassign)
                // }
            }  //if ($this->iassign->activity_group == 0)
            else {

                // if ($iassign_list) {
                foreach ($iassign_list as $iassign) {
                    if ($iassign->type_iassign == 3) {
                        $iassign_array_exercise[$i_exercise] = $iassign;
                        $i_exercise++;
                    } //if ($iassign->type_iassign == 3)
                    if ($iassign->type_iassign == 2) {
                        $iassign_array_test[$i_test] = $iassign;
                        $i_test++;
                    } //if ($iassign->type_iassign == 2)
                    if ($iassign->type_iassign == 1) {
                        $iassign_array_example[$i_example] = $iassign;
                        $i_example++;
                    } //if ($iassign->type_iassign == 1)
                } //foreach ($iassign_list as $iassign)
                //}
            }

            if ($iassign_array_exercise) {
                $title = get_string('exercise', 'iassign');
                $this->show_iassign($title, $iassign_array_exercise, $i_exercise);
            } // if ($iassign_array_exercise)
            if ($iassign_array_test) {
                $title = get_string('test', 'iassign');
                $this->show_iassign($title, $iassign_array_test, $i_test);
            } // if ($iassign_array_test)
            if ($iassign_array_example) {
                $title = get_string('example', 'iassign');
                $this->show_iassign($title, $iassign_array_example, $i_example);
            } // if ($iassign_array_example)
            if ($iassign_array_general) {
                $title = "";
                $this->show_iassign($title, $iassign_array_general, $i_general);
            } //if ($iassign_array_general)
        } else {
            echo $OUTPUT->notification(get_string('no_activity', 'iassign'), 'notifysuccess');
        }
        if (count($iassign_list) > 5 and !(has_capability('mod/iassign:submitiassign', $this->context, $USER->id))) {

            echo $OUTPUT->box_start();

            echo '<table width=100% border=0><tr>' . "\n";

            if (has_capability('mod/iassign:viewiassignall', $this->context, $USER->id)) {
                echo '<td width=10% align="left">' . "\n";
                echo $link_report;
                echo '</td>' . "\n";
            } // if (has_capability('mod/iassign:viewiassignall', $this->context, $USER->id))
            if (has_capability('mod/iassign:editiassign', $this->context, $USER->id)) {
                echo '<td align="left">' . "\n";
                echo $link_add;
                echo '</td>' . "\n";
            } //if (has_capability('mod/iassign:editiassign', $this->context, $USER->id))
            echo '</tr></table>' . "\n";
            echo $OUTPUT->box_end();
        } // if (count($iassign_list) > 5 and !(has_capability('mod/iassign:submitiassign', $this->context, $USER->id)))
    }

    /**
     * Display all iAssigns
     */
    function show_iassign($title, $iassign_array, $i) {
        global $USER, $CFG, $DB, $OUTPUT;

        $id = $this->cm->id;
        echo $OUTPUT->box_start();
        if (has_capability('mod/iassign:viewiassignall', $this->context, $USER->id)) {
            echo "<p><font color='#0000aa'><strong>" . $title . "</strong></font></p>";
            for ($j = 0; $j < $i; $j++) {
                $iassign_current = $iassign_array[$j]->id;
                if ($j == $i - 1)
                    $iassign_down = $iassign_array[$j]->id;
                else
                    $iassign_down = $iassign_array[$j + 1]->id;
                if ($j > 0)
                    $iassign_up = $iassign_array[$j - 1]->id;
                else
                    $iassign_up = $iassign_array[$j]->id;
                if ($iassign_array[$j]->visible == 0)
                    $links = "&nbsp;<a href='view.php?id=$id&userid_iassign=$USER->id&action=view&iassign_current=$iassign_current'><font color='#bbbbbb'>" . $iassign_array[$j]->name . "</font></a>";
                else
                    $links = "&nbsp;<a href='view.php?id=$id&userid_iassign=$USER->id&action=view&iassign_current=$iassign_current'>" . $iassign_array[$j]->name . "</a>";
                if (has_capability('mod/iassign:editiassign', $this->context, $USER->id) and $USER->iassignEdit == 1) {
                    $aux = "&id=$id&iassign_current=$iassign_current&iassign_up=$iassign_up&iassign_down=$iassign_down";
                    $link_up = "&nbsp;<a href='view.php?action=up$aux'>" . icons::insert('move_up_iassign') . "</a>";
                    $link_down = "&nbsp;<a href='view.php?action=down$aux'>" . icons::insert('move_down_iassign') . "</a>";
                    $link_delete = "&nbsp;<a href='view.php?action=delete$aux'>" . icons::insert('delete_iassign') . "</a>";
                    $link_visible_hide = "&nbsp;<a href='view.php?action=visible$aux'>" . icons::insert('hide_iassign') . "</a>";
                    $link_visible_show = "&nbsp;<a href='view.php?action=visible$aux'>" . icons::insert('show_iassign') . "</a>";
                    $link_edit = "&nbsp;<a href='view.php?action=edit$aux'>" . icons::insert('edit_iassign') . "</a>";
                    if (count($iassign_array) > 1) {
                        if ($j == 0)
                            $links .=$link_down;
                        elseif ($j == $i - 1)
                            $links .=$link_up;
                        else
                            $links .=$link_up . $link_down;
                    } //if (count($iassign_array) > 1)
                    $links .=$link_edit . $link_delete;

                    if ($iassign_array[$j]->visible == 0)
                        $links .= $link_visible_show;
                    else
                        $links .= $link_visible_hide;
                } //if ($USER->iassignEdit == 1 && has_capability('mod/iassign:editiassign', $this->context, $USER->id))
                echo '<p>' . $links . '</p>' . "\n";
            }
        } //if (has_capability('mod/iassign:viewiassignall', $this->context, $USER->id))
        elseif (has_capability('mod/iassign:submitiassign', $this->context, $USER->id)) { //student
            echo '<table width=100% ><tr>' . "\n";
            echo "<td width=70% align='left'><font color='#0000aa'><strong>" . $title . "</strong></font></td>" . "\n";
            echo '</tr></table>' . "\n";

            for ($j = 0; $j < $i; $j++) {
                $icon_status = "";
                $icon_comment = "";
                if ($iassign_array[$j]->visible == 1) {
                    $iassign_current = $iassign_array[$j]->id;
                    $iassign_submission = $DB->get_record('iassign_submission', array('iassign_statementid' => $iassign_current, 'userid' => $USER->id));
                    $links = "&nbsp;<a href='view.php?id=$id&userid_iassign=$USER->id&action=view&iassign_current=$iassign_current'>" . $iassign_array[$j]->name . "</a>";
                    $icon_status = "";
                    $icon_comment = "";
                    if ($iassign_submission) {

                        //receiver=1 - message to teacher
                        //receiver=2 - message to student

                        /*
                          $verify_message = $DB->get_record_sql("SELECT COUNT(iassign_submissionid)
                          FROM {$CFG->prefix}ia_assign_submissions_comment
                          WHERE iassign_submissionid = '$iassign_submission->id'
                          and  return_status= 0 and receiver=2");
                         */

                        $params = array('iassign_submissionid' => $iassign_submission->id, 'return_status' => '0', 'receiver' => '2');
                        $verify_message = $DB->get_record_sql('SELECT COUNT(iassign_submissionid)
                             FROM {iassign_submission_comment}
                             WHERE iassign_submissionid = :iassign_submissionid
                             AND return_status= :return_status
                             AND receiver= :receiver', $params); // " - jed/emacs

                        if ($verify_message)
                            foreach ($verify_message as $tmp)
                                $sum_comment = $tmp;

                        if ($sum_comment > 0)
                            $icon_comment = icons::insert('comment_unread');

                        if ($iassign_array[$j]->type_iassign == 3) {
                            if ($iassign_array[$j]->show_answer == 1) {
                                if ($iassign_submission->status == 3)
                                    $icon_status = icons::insert('correct');
                                elseif ($iassign_submission->status == 2)
                                    $icon_status = icons::insert('incorrect');
                                elseif ($iassign_submission->status == 1)
                                    $icon_status = icons::insert('post');
                                elseif ($iassign_submission->status == 0)
                                    $icon_status = icons::insert('not_post');
                            }//if ($iassign_array[$j]->show_answer==1)
                            else {
                                if ($iassign_submission->status == 0)
                                    $icon_status = icons::insert('not_post');
                                else
                                    $icon_status = icons::insert('post');
                            }
                        } //if ($iassign_array[$j]->type_iassign == 3)
                    } //if ($iassign_submission)
                    elseif ($iassign_array[$j]->type_iassign == 3) {
                        $icon_status = icons::insert('not_post');
                    } //if ($iassign_array[$j]->type_iassign == 3)
                    echo '<p>' . $icon_status . '&nbsp;' . $links . '&nbsp;' . $icon_comment . '</p>' . "\n";
                } //if ($iassign_array[$j]->visible == 1)
            } //for ($j = 0; $j < $i; $j++)
        } //elseif (has_capability('mod/iassign:submitiassign', $this->context, $USER->id))
        else {
            print_string('no_permission_iassign', 'iassign');
        }
        echo $OUTPUT->box_end();
    }

// show_iassign($title, $iassign_array, $i)

    /**
     * Show message of return
     */
    function return_home_course($message) {
        global $DB, $OUTPUT;
        $link_return = "&nbsp;<a href='" . $this->return . "'>" . icons::insert('home') . get_string('activities_page', 'iassign') . "</a>";
        echo $OUTPUT->box_start();
        echo '<table width=100% border=0 valign="top">' . "\n";
        echo '<tr><td align="left"><strong>' . "\n";
        print_string($message, 'iassign');
        echo '</strong></td>' . "\n";
        echo '<td width=20% align="right">' . "\n";
        echo $link_return;
        echo '</td></tr></table>' . "\n";
        echo $OUTPUT->box_end();
        // echo $OUTPUT->footer();
        die;
    }

    /**
     * Search comment of activity
     */
    function search_comment_submission($iassign_submissionid) {
        global $USER, $DB, $OUTPUT, $COURSE;
        $context = get_context_instance(CONTEXT_COURSE, $COURSE->id);

        $comments = $DB->get_records_list('iassign_submission_comment', 'iassign_submissionid', array('iassign_submissionid' => $iassign_submissionid), 'timecreated DESC'); //'ORDER BY "timecreated" ASC'
        $text = "";
        if ($comments) {

            foreach ($comments as $tmp) {
                $user_answer = $DB->get_record("user", array('id' => $tmp->comment_authorid));
                if (has_capability('mod/iassign:editiassign', $context, $tmp->comment_authorid)) {
                    $text .="<tr><td bgcolor='#fee7ae'><b> $user_answer->firstname</b>&nbsp;(" . userdate($tmp->timecreated) . "</br>";
                    $text .="$tmp->comment</td></tr>";
                } else {
                    $text .="<tr><td bgcolor='#dce7ec'>&raquo;<b> $user_answer->firstname</b>&nbsp;(" . userdate($tmp->timecreated) . "</br>";
                    $text .="$tmp->comment</td></tr>";
                }
            } //foreach ($comments as $tmp)
        }
        return $text;
    }

    /**
     * Update comment of activity
     */
    function update_comment($iassign_submissionid) {
        global $USER, $DB, $OUTPUT;
        if (!has_capability('mod/iassign:submitiassign', $this->context, $USER->id))
            $receiver = 1; // student message to teacher
        else
            $receiver = 2;  // teacher message to student

        $verify_message = $DB->get_records('iassign_submission_comment', array('iassign_submissionid' => $iassign_submissionid)); //

        if ($verify_message) {
            foreach ($verify_message as $message) {
                if ($message->receiver == $receiver) {
                    $newentry = new stdClass();
                    $newentry->id = $message->id;
                    $newentry->return_status = 1;
                    if (!$DB->update_record('iassign_submission_comment', $newentry)) {
                        print_error('error_update', 'iassign');
                    } // if (!$DB->update_record('iassign_submission_comment', $newentry))
                } //if ($message->receiver == $receiver)
                add_to_log($this->course->id, "iassign", "update comment", "view.php?id={$this->iassign->id}", $iassign_submissionid, $this->cm->id, $USER->id);
            } //foreach ($verify_message as $message)
        }
    }

    /**
     * Record comment of activity
     */
    function write_comment_submission() {
        global $USER, $CFG, $DB, $OUTPUT;
        $id = $this->cm->id;
        $submission_comment = optional_param('submission_comment', NULL, PARAM_TEXT);
        $row = optional_param('row', 0, PARAM_INT);
        $column = optional_param('column', 0, PARAM_INT);

        $sum_comment = 0;

        $return = "" . $CFG->wwwroot . "/mod/iassign/view.php?action=viewsubmission&id=" . $id . "&iassign_submission_current=" . $this->iassign_submission_current . "&userid_iassign=" . $this->userid_iassign . "&iassign_current=" . $this->activity->get_activity()->id . "&row=" . ($row) . "&column=" . ($column);

        $link_return = "&nbsp;<a href='" . $return . "'>" . icons::insert('return_home') . get_string('return', 'iassign') . "</a>";

        $str1 = trim($submission_comment);
        $str2 = trim(get_string('box_comment_message', 'iassign'));

        if (!empty($submission_comment) and (strcmp($str1, $str2) != 0 )) {
            if (has_capability('mod/iassign:submitiassign', $this->context, $USER->id)) {
                $receiver = 1; // student message to teacher
                $this->action = 'view';
            } else {
                $receiver = 2;  // teacher message to student
                $this->action = 'viewsubmission';
            } //if (has_capability('mod/iassign:submitiassign', $this->context, $USER->id))


            $iassign_submission = $DB->get_record("iassign_submission", array("id" => $this->iassign_submission_current));

            if (!$iassign_submission) {
                $iassign_statement = $DB->get_record("iassign_statement", array("id" => $this->activity->get_activity()->id));

                $id_submission = $this->new_submission($iassign_statement->id, $this->userid_iassign, $receiver);
                $this->iassign_submission_current = $id_submission;
            } else {
                $id_submission = $iassign_submission->id;
            }// if (!$iassign_submission)
            // $comments = $DB->get_record_sql("SELECT COUNT(iassign_submissionid) FROM {$CFG->prefix}ia_assign_submissions_comment
            //             WHERE iassign_submissionid = '$id_submission' and comment='$submission_comment' and comment_authorid='$USER->id'"); //
            //Attention: this Moodle function 'get_record_sql' makes a replace in ':comment'
            $params = array("iassign_submissionid" => $id_submission, "comment" => $submission_comment, "comment_authorid" => $USER->id);
            $comments = $DB->get_record_sql('SELECT COUNT(iassign_submissionid)
                        FROM {iassign_submission_comment}
                        WHERE iassign_submissionid = :iassign_submissionid AND comment= :comment
                        AND comment_authorid= :comment_authorid', $params); // " - jed/emacs

            if ($comments)
                foreach ($comments as $tmp)
                    $sum_comment = $tmp;

            if ($sum_comment == 0) {
                $newentry = new stdClass();
                $newentry->iassign_submissionid = $id_submission;
                $newentry->comment_authorid = $USER->id;
                $newentry->timecreated = time();
                $newentry->comment = $submission_comment;
                $newentry->receiver = $receiver;
                $ia_assign_submissions_comment_id = $DB->insert_record('iassign_submission_comment', $newentry);

                add_to_log($this->course->id, "iassign", "add comment", "view.php?id={$this->iassign->id}", $ia_assign_submissions_comment_id, $this->cm->id, $USER->id);
            }
        } //if (!empty($submission_comment) and (strcmp($str1, $str2) != 0))
        // if ($this->action=='viewsubmission'){
        //   echo $OUTPUT->header();
        //   $this->return_last('confirm_add_comment', $link_return);
        //   die;
        //   }

        return true;
    }

    /**
     * Writes a new submission
     */
    function new_submission($iassignid, $id_user, $receiver) {
        global $USER, $DB, $OUTPUT;
        $newentry = new stdClass();
        $newentry->iassign_statementid = $iassignid;
        $newentry->userid = $id_user;
        $newentry->timecreated = time();
        $newentry->timemodified = time();
        $newentry->answer = 0; // student only submit message
        if ($receiver == 2)// teacher message to student (write id teacher)
            $newentry->teacher = $USER->id;

        if (!$newentry->id = $DB->insert_record("iassign_submission", $newentry))
            return_home_course(get_string('error_insert_submissions', 'iassign'));
        else
            add_to_log($this->course->id, "iassign", "add submission", "view.php?id={$this->iassign->id}", $newentry->id, $this->cm->id, $USER->id);

        return $newentry->id;
    }

    /**
     * Return to a specific address of page
     */
    function return_last($message, $link_return) {
        global $DB, $OUTPUT;
        echo $OUTPUT->box_start();
        echo '<table width=100% border=0 valign="top">' . "\n";
        echo '<tr><td align="left"><strong>' . "\n";
        print_string($message, 'iassign');
        echo '</strong></td>' . "\n";
        echo '<td width=20% align="right">' . "\n";
        echo $link_return;
        echo '</td></tr></table>' . "\n";
        echo $OUTPUT->box_end();
        echo $OUTPUT->footer();
        die;
    }

}

// class iassign
// if ($iassign_statement->type_iassign == 3)

/**
 * Class to manage Interactive Learning Module (iLM)
 */
class ilm {

    var $ilm;

    function ilm($id) {
        global $DB;
        $this->ilm = $DB->get_record("iassign_ilm", array("id" => $id));
        if (empty($this->ilm))
            $this->ilm = null;
    }

    /**
     * Shows activity in iLM
     * */
    function view_iLM($iassign_statement, $student_answer, $enderecoPOST, $view) {
        global $USER, $CFG, $COURSE, $DB, $OUTPUT;

        $notSEND = 'true'; // $notSEND='false';
        //Search of iLM data used in the current activity

        $file_jar = $this->ilm->file_jar;
        $file_class = $this->ilm->file_class;
        $width = $this->ilm->width;
        $height = $this->ilm->heigth;

        $special_param1 = $iassign_statement->special_param1;


        //ATENCAO: Patricia, com essa linha o Epiphany nao consegue abrir conteudos...
        //
        //Security iLM: remove the entry
        // $DB->delete_records("iassign_security", array("iassign_statementid" => $iassign_statement->id, "userid" => $USER->id));

        /*
         * $view==true  - file with exercise
         * $view==false - exercise submitted by the student (search database)
         */

        $this->context = get_context_instance(CONTEXT_MODULE, $USER->cm);

        $fs = get_file_storage();
        $files = $fs->get_area_files($this->context->id, 'mod_iassign', 'activity', $iassign_statement->file);
        if ($files) {
            foreach ($files as $value) {
                if ($value->get_filename() != '.')
                    $fileid = $value->get_id();
            }
        }


        if ($this->ilm->name == "iGeom") { // is iGeom exercise of script
            $file = $fileid; //$iassign_statement->file;
            if ($special_param1 == 1 and !empty($student_answer) and (!$view))
                $view = 1;
            elseif (!$view)
                $file = $student_answer;
        } else {
            if ($view)
            //$file = $this->context->id;
                $file = $fileid; //$iassign_statement->file;
            else
                $file = $student_answer;
        }




        $id_iLM_security = $this->write_iLM_security($iassign_statement->id, addslashes($file));
        $iassign_iLM_security = $DB->get_record("iassign_security", array("id" => $id_iLM_security));
        $token = md5($iassign_iLM_security->timecreated);
        $end_file = $CFG->wwwroot . DIRECTORY_SEPARATOR . 'mod' . DIRECTORY_SEPARATOR . 'iassign' . DIRECTORY_SEPARATOR . 'ilm_security.php?id=' . $id_iLM_security . '&token=' . $token . '&view=' . $view;

        $iassign = "
        <script type='text/javascript'>
          //<![CDATA[
         function resp () {
           var strAnswer = document.applets[0].getAnswer();
           var value = document.applets[0].getEvaluation();
           var comment = document.formEnvio.submission_comment.value;
           if ((strAnswer==null || strAnswer=='' || strAnswer==-1) && (comment==null || comment=='')){ // undefined
              alert('" . get_string('activity_empty', 'iassign') . "');
              return false; // sem erro
              }
           else {
              document.formEnvio.MA_POST_Archive.value = strAnswer;
              document.formEnvio.MA_POST_Value.value = value;
              document.formEnvio.submit();
              return true; // com erro
              }
           }
       //]]>
       </script>"; //

        $lang = substr(current_language(), 0, 2);
        $iassign .= "<center><form name='formEnvio' id='formEnvio' method='post' action='$enderecoPOST' enctype='multipart/form-data'>";

        $iassign .= "<applet name='iLM' codebase='./ilm/' code='$file_class' archive='$file_jar' width='$width' height='$height' vspace=10 hspace=10>
        <param name='MA_PARAM_PropositionURL' value='true'/>
        <param name='MA_PARAM_notSEND' value='$notSEND'/>
         <param name='lang' value='$lang'/>
         <param name='MA_PARAM_addresPOST' value='$enderecoPOST'> <!-- 2 -->
         <param name='MA_PARAM_Proposition' value='" . $end_file . "'>" . $this->special_param($special_param1, $student_answer) . "
         </applet> <br/>
        <input type='hidden' name='MA_POST_Archive'>
        <input type='hidden' name='MA_POST_Value'>";

        if (!has_capability('mod/iassign:evaluateiassign', $USER->context, $USER->id))
            $iassign .="<p><textarea rows='2' cols='60' name='submission_comment'></textarea></p>";
        else
            $iassign .="<input type='hidden' name='submission_comment'>";

        if ($USER->iassignEdit == 1)
            $iassign .= "<center>
            <input type=button value='" . get_string('submit_iassign', 'iassign') . "' onClick = 'resp()'
                 title='" . get_string('message_submit_iassign', 'iassign') . "'>
            </center>";

        $iassign .= "</form></center>";
        return $iassign;
    }

    /**
     * inserts the parameter that displays special algorithm for sending through of iLM
     * feature specific to iLM iGeom
     */
    function special_param($special_param1, $student_answer) {
        if ($special_param1 == 1) {
            // is iGeom exercise of script
            $param = "\n         <param name='TIPO_SCRIPT' value='1'>\n" .
                    "         <param name='BOTAOSCR1' value='" . $student_answer . "' />";
        }
        else
            $param = "";
        return $param;
    }

    function write_iLM_security($iassign_statementid, $file) {
        global $CFG, $USER, $COURSE, $DB, $OUTPUT;
        $newentry = new stdClass();
        $newentry->iassign_statementid = $iassign_statementid;
        $newentry->userid = $USER->id;
        $newentry->file = $file;
        $newentry->timecreated = time();
        $newentry->view = 1;
        $id_iLM_security = $DB->insert_record("iassign_security", $newentry);
        if (!$id_iLM_security) {
            print_error('error_security', 'iassign');
        } //if (!$DB->insert_record("iassign_security", $newentry))

        return $id_iLM_security;
    }

    /**
     * Confirms the existence of the jar file
     */
    function confirms_jar_copia($fileid, $file_jar) {
        global $USER, $CFG, $DB, $OUTPUT;

        //verifica se o arquivo jar existe
        $pathfilejar = $CFG->dirroot . "/mod/iassign/ilm/" . $file_jar;
        if (!file_exists($pathfilejar)) {
            echo $OUTPUT->notification(get_string('error_confirms_jar', 'iassign'), 'notifysuccess');
            return false;
        }

        //verifica se o arquivo jar está cadastrado no BD
        $iassign_ilm = $DB->get_record('iassign_ilm', array('file_jar' => $file_jar));
        if (!$iassign_ilm) {
            echo $OUTPUT->notification(get_string('error_confirms_ilm', 'iassign'), 'notifysuccess');
            return false;
        }


        $fs = get_file_storage();
        $file = $fs->get_file_by_id($fileid);

        //verifica se o arquivo da atividade existe
        if (!$file) {
            echo $OUTPUT->notification(get_string('error_view_ilm', 'iassign'), 'notifysuccess');
            return false;
        }

        //verifica se o arquivo é do tipo do ilm

        $fileiassign = $file->get_filename();
        $extension = explode(".", $fileiassign);
        if ($iassign_ilm->extension != $extension[1]) {
            echo $OUTPUT->notification(get_string('error_confirms_gab', 'iassign'), 'notifysuccess');
            return false;
        }
        return true;
    }

    /**
     * Confirms the existence of the jar file
     */
    function confirms_jar($itemid, $file_jar, $cmid) {
        global $USER, $CFG, $DB, $OUTPUT;

        $context = get_context_instance(CONTEXT_MODULE, $cmid);


        //verifica se o arquivo jar existe
        $pathfilejar = $CFG->dirroot . "/mod/iassign/ilm/" . $file_jar;
        if (!file_exists($pathfilejar)) {
            echo $OUTPUT->notification(get_string('error_confirms_jar', 'iassign'), 'notifysuccess');
            return false;
        }

        //verifica se o arquivo jar está cadastrado no BD
        $iassign_ilm = $DB->get_record('iassign_ilm', array('file_jar' => $file_jar));
        if (!$iassign_ilm) {
            echo $OUTPUT->notification(get_string('error_confirms_ilm', 'iassign'), 'notifysuccess');
            return false;
        }

        $fs = get_file_storage();
        $files = $fs->get_area_files($context->id, 'mod_iassign', 'activity', $itemid);
        $file = null;
        foreach ($files as $value) {
            if ($value->get_filename() != '.')
                $file = $value;
        }

        // $file=$fs->get_file_by_id($fileid);
        //verifica se o arquivo da atividade existe
        if (!$file) {
            echo $OUTPUT->notification(get_string('error_view_ilm', 'iassign'), 'notifysuccess');
            return false;
        }

        //verifica se o arquivo é do tipo do ilm

        $fileiassign = $file->get_filename();
        $extension = explode(".", $fileiassign);
        if ($iassign_ilm->extension != $extension[1]) {
            echo $OUTPUT->notification(get_string('error_confirms_gab', 'iassign'), 'notifysuccess');
            return false;
        }
        return true;
    }

}

class activity {

    var $activity;

    function activity($id) {
        global $DB;
        $this->activity = $DB->get_record("iassign_statement", array("id" => $id));
        if (empty($this->activity))
            $this->activity = null;
    }

    function get_activity() {
        if ($this->activity != null)
            return $this->activity;
        else
            return null;
    }

    /**
     * Delete interactive activities
     */
    function delete($return) {
        global $USER, $CFG, $DB, $OUTPUT;

        $iassign_submission_currents = $DB->get_records("iassign_submission", array("iassign_statementid" => $this->activity->id));

        $output = $OUTPUT->header();
        $output .= $OUTPUT->box_start();
        $output .= "<p>" . get_string('delete_activity', 'iassign') . "&nbsp;<strong>" . $this->activity->name . "</strong></p>";
        if ($iassign_submission_currents) {
            $output .= "<p>" . get_string('number_submissions', 'iassign') . "&nbsp;<strong>" . count($iassign_submission_currents) . "</strong></p>";
            if (!has_capability('mod/iassign:deleteiassignnotnull', $USER->context, $USER->id)) {
                $output .= $OUTPUT->heading(get_string('delete_activity_permission_adm', 'iassign'));
                $output .= $OUTPUT->single_button($return, get_string('return', 'iassign'), 'get');
                $output .= $OUTPUT->box_end();
                $output .= $OUTPUT->footer();
                echo $output;
                die;
            } //if (!has_capability('mod/iassign:deleteiassignnotnull', $this->context, $USER->id))
        } // if ($iassign_submission_currents)
        else
            $output .= "<p>" . get_string('not_submissions_activity', 'iassign') . "</p>";
        $output .= '<table width=50% border=0>';
        $output .= '<tr valign="top"><td>';
        $output .= "<p>" . get_string('what_do', 'iassign') . "</p>";
        $output .= '</td><td>';

        $bottonDelete_yes = get_string('delete_iassign', 'iassign');
        $deleteiassignyes = $CFG->wwwroot . "/mod/iassign/view.php?id=" . $USER->cm . "&action=deleteyes&iassign_current=" . $this->activity->id;
        $output .= "<form name='formDelete' id='formDelete' method='post' action='$deleteiassignyes' enctype='multipart/form-data'>";
        $output .= " <input type=submit value='$bottonDelete_yes'/>\n";
        $output .= "</form>\n";
        $output .= '</td><td>';
        $bottonDelete_no = get_string('delete_cancel', 'iassign');
        $deleteiassignno = $CFG->wwwroot . "/mod/iassign/view.php?id=" . $USER->cm . "&action=deleteno&iassign_current=" . $this->activity->id;
        $output .= "<form name='formDelete' id='formDelete' method='post' action='$deleteiassignno' enctype='multipart/form-data'>\n";
        $output .= " <p><input type=submit value='$bottonDelete_no'/></p>\n";
        $output .= "</form>\n";
        $output .= '</td></tr></table>' . "\n";
        $output .= $OUTPUT->box_end();
        $output .= $OUTPUT->footer();
        echo $output;
    }

    function deleteno($return) {
        global $OUTPUT;
        $output = $OUTPUT->header();
        $output .= $OUTPUT->heading(get_string('confirm_not_delete_iassign', 'iassign'));
        $output .= $OUTPUT->single_button($return, get_string('return', 'iassign'), 'get');
        $output .= $OUTPUT->footer();
        echo $output;
    }

    function deleteyes($return) {
        global $USER, $CFG, $DB, $OUTPUT;
        $msg = '';

        if (!empty($this->activity->id)) {
            $iassign_submission_currents = $DB->get_records("iassign_submission", array("iassign_statementid" => $this->activity->id));
            if ($iassign_submission_currents) {
                if (has_capability('mod/iassign:deleteassignnull', $USER->context, $USER->id)) {
                    foreach ($iassign_submission_currents as $iassign_submission)
                        $DB->delete_records('iassign_submission_comment', array('iassign_submissionid' => $iassign_submission->id));
                    $delete_iassign_submission_currents = $DB->delete_records("iassign_submission ", array("iassign_statementid" => $this->activity->id));
                } // if ($iassign_submission_currents)
            }

            $this->delete_calendar($this->activity->id);
            $delete_iassign_current = $DB->delete_records('iassign_statement', array('id' => $this->activity->id));
            iassign::update_grade_iassign($this->activity->iassignid);

            if ($delete_iassign_current) {
                $msg = get_string('confirm_delete_iassign', 'iassign');
            } else {
                $msg = get_string('error_confirm_delete_iassign', 'iassign');
            }
            // if (($this->action == 'deleteyes') and (has_capability('mod/iassign:deleteassignnull', $this->context, $USER->id)))
        }
        $output = $OUTPUT->header();
        $output .= $OUTPUT->heading($msg);
        $output .= $OUTPUT->single_button($return, get_string('return', 'iassign'), 'get');
        $output .= $OUTPUT->footer();
        echo $output;
    }

    //function delete_iassign ()

    /**
     * Changes position within of interactive group activity
     */
    function move_iassign($target, $return) {
        global $DB, $OUTPUT;
        $iassign_target = $DB->get_record("iassign_statement", array("id" => $target));
        $aux = $this->activity->position;
        $newentry = new stdClass();
        $newentry->id = $this->activity->id;
        $newentry->position = $iassign_target->position;

        if (!$DB->update_record('iassign_statement', $newentry)) {
            print_error('error_update_move_iassign', 'iassign');
        }

        $newentry->id = $iassign_target->id;
        $newentry->position = $aux;
        if (!$DB->update_record('iassign_statement', $newentry))
            print_error('error_update_move_iassign', 'iassign');
        redirect($return);
    }

    /**
     * Enable or disable the display of interactive activities
     */
    function visible_iassign($return) {
        global $DB;
        $newentry = new stdClass();
        $newentry->id = $this->activity->id;
        $newentry->visible = $this->activity->visible == 0 ? 1 : 0;
        if (!$DB->update_record('iassign_statement', $newentry))
            print_error('error_update_visible_iassign', 'iassign');
        redirect($return);
    }

    /**
     * Add news interactive activities
     */
    function new_iassign($param) {
        global $DB;

        $newentry = new stdClass();
        $newentry->iassignid = $param->iassignid;
        $newentry->name = $param->name;
        $newentry->type_iassign = $param->type_iassign;
        $newentry->proposition = $param->proposition;
        $newentry->author_name = $param->author_name;
        $newentry->author_modified_name = $param->author_modified_name;
        $newentry->iassign_ilmid = $param->iassign_ilmid;
        $newentry->file = $param->file;
        $newentry->grade = $param->grade;
        $newentry->timemodified = time();
        $newentry->timecreated = time();
        if ($param->type_iassign == 1) {
            $newentry->timedue = 0;
            $newentry->timeavailable = 0;
        } //if ($param->type_iassign == 1)
        else {
            $newentry->timedue = $param->timedue;
            $newentry->timeavailable = $param->timeavailable;
        }
        $newentry->preventlate = $param->preventlate;
        $newentry->test = $param->test;
        $newentry->special_param1 = $param->special_param1;
        $newentry->visible = $param->visible;
        $newentry->position = $param->position;
        $newentry->max_experiment = $param->max_experiment;
        $newentry->dependency = $param->dependency;
        $newentry->automatic_evaluate = $param->automatic_evaluate;
        $newentry->show_answer = $param->show_answer;

        if ($id = $DB->insert_record("iassign_statement", $newentry)) {

            $component = 'mod_iassign';
            $filearea = 'activity';
            $fs = get_file_storage();
            $file = $fs->get_file_by_id($param->file);
            $itemid = $file->get_itemid() + $id;
            $newfile = $fs->create_file_from_storedfile(array('contextid' => $param->context->id,
                'component' => $component,
                'filearea' => $filearea,
                'itemid' => $itemid), $file);
            $updateentry = new stdClass();
            $updateentry->id = $id;
            $updateentry->file = $newfile->get_itemid();
            if (!$DB->update_record("iassign_statement", $updateentry))
                print_error('error_add', 'iassign');

            if ($param->type_iassign == 3)
                iassign::update_grade_iassign($param->iassignid);
            return $id;
        }else
            print_error('error_add', 'iassign');
    }

    /**
     * Add the calendar entries for this iassign
     *
     * @param int $coursemoduleid - Required to pass this in because it might not exist in the database yet
     * @return bool
     */
    static function add_calendar($iassignid) {
        global $DB, $CFG;
        require_once($CFG->dirroot . '/calendar/lib.php');

        $iassign_statement = $DB->get_record("iassign_statement", array("id" => $iassignid));
        $iassign = $DB->get_record("iassign", array("id" => $iassign_statement->iassignid));

        $event = new stdClass();
        $event->name = $iassign->name . '&nbsp;-&nbsp;' . $iassign_statement->name;
        $event->description = $iassign_statement->name;
        $event->courseid = $iassign->course;
        $event->groupid = 0;
        $event->userid = 0;
        $event->modulename = 'iassign';
        $event->instance = $iassign->id;
        $event->eventtype = 'due';
        $event->timestart = $iassign_statement->timeavailable;
        $event->timeduration = ($iassign_statement->timedue - $iassign_statement->timeavailable);
        calendar_event::create($event);
    }

    /**
     * Update the calendar entries for this iassign
     *
     * @param int $coursemoduleid - Required to pass this in because it might not exist in the database yet
     * @return bool
     */
    function update_calendar($iassignid, $olddescription) {
        global $DB, $CFG;
        require_once($CFG->dirroot . '/calendar/lib.php');

        $iassign_statement = $DB->get_record("iassign_statement", array("id" => $iassignid));
        $iassign = $DB->get_record("iassign", array("id" => $iassign_statement->iassignid));

        $event = new stdClass();
        $event->id = 0;
        $events = $DB->get_records('event', array('modulename' => 'iassign', 'instance' => $iassign->id));
        if ($events) {
            foreach ($events as $value) {
                if ($value->description == $olddescription) {
                    $event->id = $value->id;
                }
            }
        }
        if ($event->id != 0) {
            $event->name = $iassign->name . '&nbsp;-&nbsp;' . $iassign_statement->name;
            $event->description = $iassign_statement->name;
            $event->timestart = $iassign_statement->timeavailable;
            $event->timeduration = ($iassign_statement->timedue - $iassign_statement->timeavailable);

            $calendarevent = calendar_event::load($event->id);
            $calendarevent->update($event);
        }
        else
            $this->add_calendar($iassignid);
    }

    /**
     * Update the calendar entries for this iassign
     *
     * @param int $coursemoduleid - Required to pass this in because it might not exist in the database yet
     * @return bool
     */
    function delete_calendar($iassignid) {
        global $DB, $CFG;
        require_once($CFG->dirroot . '/calendar/lib.php');

        $iassign_statement = $DB->get_record("iassign_statement", array("id" => $iassignid));
        $iassign = $DB->get_record("iassign", array("id" => $iassign_statement->iassignid));
        $events = $DB->get_records('event', array('modulename' => 'iassign', 'instance' => $iassign->id));
        if ($events) {
            foreach ($events as $value) {
                if ($value->description == $iassign_statement->name) {
                    $DB->delete_records('event', array('id' => $value->id));
                }
            }
        }
    }

    /**
     * Update interactive activities
     */
    function update_iassign($param) {
        global $DB;

        $component = 'mod_iassign';
        $filearea = 'activity';
        $fs = get_file_storage();
        $file = $fs->get_file_by_id($param->file);
        $fileold = $fs->get_file_by_id($param->fileold);

        if ($param->file != $param->fileold) {

            if (!$fs->file_exists($param->context->id, $component, $filearea, $file->get_itemid(), $file->get_filepath(), $file->get_filename())) {
                $itemid = $file->get_itemid() + $param->iassign_id;
                $newfile = $fs->create_file_from_storedfile(array('contextid' => $param->context->id,
                    'component' => $component,
                    'filearea' => $filearea,
                    'itemid' => $itemid), $file);
                $param->file = $newfile->get_itemid();
            }else
                $param->file = $file->get_itemid();


            if ($fileold) {
                $fileoldarea = $fs->get_area_files($fileold->get_contextid(), $fileold->get_component(), $fileold->get_filearea(), $fileold->get_itemid());
                foreach ($fileoldarea as $value) {
                    $value->delete();
                }
            }
        } else {
            $param->file = $file->get_itemid();
        }


        $newentry = new stdClass();
        $newentry->id = $param->iassign_id;
        $newentry->name = $param->name;
        $newentry->type_iassign = $param->type_iassign;
        $newentry->proposition = $param->proposition;
        $newentry->iassign_ilmid = $param->iassign_ilmid;

        $newentry->file = $param->file;
        $newentry->grade = $param->grade;
        $newentry->author_modified_name = $param->author_modified_name;

        $newentry->timemodified = time();
        if ($param->type_iassign == 1) {
            $newentry->timedue = 0;
            $newentry->timeavailable = 0;
        } //if ($param->type_iassign == 1)
        else {
            $newentry->timedue = $param->timedue;
            $newentry->timeavailable = $param->timeavailable;
        }
        $newentry->preventlate = $param->preventlate;
        $newentry->test = $param->test;
        $newentry->special_param1 = $param->special_param1;
        $newentry->visible = $param->visible;
        $newentry->max_experiment = $param->max_experiment;
        $newentry->dependency = $param->dependency;
        $newentry->automatic_evaluate = $param->automatic_evaluate;
        $newentry->show_answer = $param->show_answer;

        if (!$DB->update_record("iassign_statement", $newentry))
            print_error('error_update', 'iassign');

        if ($param->type_iassign == 3) {
            iassign::update_grade_iassign($param->iassignid);
        }

        return $newentry->id;
        // if ($param->type_iassign==3)
        //  $this->update_grade_iassign($param->iassignid);
    }

    function show_info_iassign() {
        global $DB, $OUTPUT;

        $output = '<p><strong>' . get_string('proposition', 'iassign') . ':</strong>&nbsp;' . $this->activity->proposition . '</p>' . "\n";
        if ($this->activity->type_iassign == 3) {
            if ($this->activity->dependency == 0) {
                $output .= '<p><strong>' . get_string('independent_activity', 'iassign') . '</strong></p>' . "\n";
            } else {
                $dependencys = explode(';', $this->activity->dependency);
                $output .= '<p><strong>' . get_string('dependency', 'iassign') . '</strong></p>';
                foreach ($dependencys as $dependency) {
                    $dependencyiassign = $DB->get_record("iassign_statement", array("id" => $dependency));
                    if ($dependencyiassign)
                        $output .= '<p>' . $dependencyiassign->name . '</p>' . "\n";
                } //foreach ($dependencys as $dependency)
            }//if ($iassign_statement->dependency == 0)
            if ($this->activity->max_experiment == 0)
                $output .= '<p><strong>' . get_string('experiment', 'iassign') . '</strong>&nbsp;' . get_string('ilimit', 'iassign');
            else
                $output .= '<p><strong>' . get_string('experiment_iassign', 'iassign') . '</strong>&nbsp;' . $this->activity->max_experiment . "\n";
            $output .= '&nbsp;&nbsp;&nbsp;<strong>' . get_string('grade_iassign', 'iassign') . '</strong>&nbsp;' . $this->activity->grade . '</p>' . "\n";
        } // if ($iassign_statement->type_iassign == 3)

        echo $OUTPUT->box($output);
    }

    /**
     * Shows date of opening and closing activities
     */
    function view_dates() {
        global $USER, $CFG, $DB, $OUTPUT;

        $return = $CFG->wwwroot . "/mod/iassign/view.php?id=" . $USER->cm;
        $link_return = "&nbsp;<a href='" . $return . "'>" . icons::insert('home') . get_string('activities_page', 'iassign') . "</a>";
        $status_iassign = "";
        $status_iassign1 = "";
        $status_iassign2 = "";
        if ($this->activity->type_iassign == 1)// activity of type example
            $type_iassign = get_string('example_iassign', 'iassign');
        elseif ($this->activity->type_iassign == 2) {// activity of type test
            $type_iassign = get_string('test_iassign', 'iassign');
            if ($this->activity->timeavailable > time()) {
                $status_iassign = get_string('previous_timeavailable', 'iassign');
            } elseif ($this->activity->timedue < time()) {
                $status_iassign = get_string('last_timedue', 'iassign');
            }
        } elseif ($this->activity->type_iassign == 3) {//activity of type exercise
            $type_iassign = get_string('exercise_iassign', 'iassign');
            if ($this->activity->timeavailable > time()) {
                $status_iassign = get_string('previous_timeavailable', 'iassign'); // before of deadline
            } elseif ($this->activity->timedue < time()) {// after delivery
                $status_iassign = get_string('last_timedue', 'iassign');
                if ($this->activity->preventlate == 1)// permitted to submit after the deadline
                    $status_iassign1 = get_string('duedate_preventlate_enable', 'iassign');
                elseif ($this->activity->preventlate == 0) {// not permitted to submit after the deadline
                    $status_iassign1 = get_string('duedate_preventlate_desable', 'iassign');
                    if ($this->activity->test == 1) // allowed to test after of deadline
                        $status_iassign2 = get_string('test_preventlate', 'iassign');
                    elseif ($this->activity->test == 0) { // not allowed to test after of deadline
                        $status_iassign2 = get_string('test_preventlate_no', 'iassign');
                    } //elseif ($iassign_statement->test == 0)
                } //elseif ($iassign_statement->preventlate == 0)
            } //elseif ($iassign_statement->timedue < time())
        } //elseif ($iassign_statement->type_iassign == 3)

        $output = '<table  width=100% ><tr>' . "\n";

//TODO duvida: como permitir ao admin,professor,monitor ver a atividade mesmo apos prazo???
//Leo testes para passar por cima com 'has_capability('mod/iassign:...', $this->context, $USER->id)
        $output .= '<td width=80%>' . $type_iassign . '</td>' . "\n";
//leo $output .= '<td width=80%>' . $type_iassign;
// $output .= $auxStr . " - status_assign=$status_iassign - this->activity->type_iassign=" . $this->activity->type_iassign. "<br/>"; // Period ended.
// $output .= '</td>' . "\n";

        if (has_capability('mod/iassign:viewiassignall', $USER->context, $USER->id) && ($this->activity->type_iassign == 3)) {
            // Link (with icon) to report survey of this batch of these insteractivy exercises
            $link_report = "<a href='" . $CFG->wwwroot . "/mod/iassign/view.php?id=" . $USER->cm . "&action=report&iassignid=" . $this->activity->iassign_ilmid . "'>" . icons::insert('view_report') . '&nbsp;' . get_string('report', 'iassign') . "</a>";
            $output .= '<td width=20% align="right">' . $link_report . '</td>' . "\n";
        } else {
            $output .= '<td width=20% align="right">' . $link_return . '</td>' . "\n";
        } //if (has_capability('mod/iassign:viewiassignall', $this->context, $USER->id) && ($iassign_statement->type_iassign == 3))
        $output .= '</tr></table>' . "\n";
        $output .= '<table  width=100% >' . "\n";
        if ($this->activity->type_iassign > 1) {
            if ($this->activity->timeavailable)
                $output .= '<tr><td width=50% align="left"> <strong>' . get_string('availabledate', 'iassign') . ':</strong>&nbsp;' . userdate($this->activity->timeavailable) . '</td>' . "\n";
            if ($this->activity->timedue)
                $output .= '<td width=50% align="left"><strong>' . get_string('duedate', 'iassign') . ':</strong>&nbsp;' . userdate($this->activity->timedue) . '</td>' . "\n";
        } //if ($iassign_statement->type_iassign > 1)
        if ($status_iassign != "" && $status_iassign1 != "" && $status_iassign2 != "")
            $output .= '<tr><td><font color="red">' . $status_iassign . '&nbsp;' . $status_iassign1 . '&nbsp;' . $status_iassign2 . '</font></td></tr>' . "\n";

        $output .= '</table>' . "\n";

        echo $OUTPUT->box($output);
    }

}

/**
 * Class to insert of icons
 */
class icons {

    static function insert($icon) {
        global $CFG;
        $string = '<img src="' . $CFG->wwwroot . '/mod/iassign/icon/' . $icon . '.gif" title="' . get_string($icon, 'iassign') . '"/>';
        return $string;
    }

}

class ilm_settings {

    static function add_edit_ilm($ilm_id, $action) {
        global $USER, $DB, $CFG;

        require_once('settings_form.php');
        $param = new object();
        $param->action = $action;
        $param->ilm_id = $ilm_id;
        $CFG->action_ilm = $action;
        $CFG->ilm_id = $ilm_id;
        if ($action == 'add') {
            $title = get_string('add_ilm', 'iassign');
            $param->title = $title;
            $param->name = "";
            $param->url = "";
            $param->description = "";
            $param->extension = "";
            $param->author = $USER->id;
            $param->file_jar = "";
            $param->file_jar_static = $param->file_jar;
            $param->file_class = "";
            $param->width = 800;
            $param->heigth = 600;
            $param->enable = 0; // 0 - hide  / 1 - show
            $param->timecreated = time();
            $param->timemodified = time();
            $param->evaluate = 0;
            $param->parent = 0;
        } elseif ($action == 'edit') {
            $iassign_ilm = $DB->get_record('iassign_ilm', array('id' => $ilm_id));
            $title = get_string('edit_ilm', 'iassign');
            if ($iassign_ilm) {
                $param->title = $title;
                $param->id = $iassign_ilm->id;
                $param->name = $iassign_ilm->name;
                $param->url = $iassign_ilm->url;
                $param->description = $iassign_ilm->description;
                $param->extension = $iassign_ilm->extension;
                $param->author = $iassign_ilm->author;
                if ($iassign_ilm->file_jar == $iassign_ilm->id)
                    $param->file_jar = '';
                else
                    $param->file_jar = $iassign_ilm->file_jar;
                $param->file_jar_static = $param->file_jar;
                $param->file_class = $iassign_ilm->file_class;
                $param->width = $iassign_ilm->width;
                $param->heigth = $iassign_ilm->heigth;
                $param->enable = $iassign_ilm->enable;
                $param->timecreated = $iassign_ilm->timecreated;
                $param->timemodified = time();
                $param->evaluate = $iassign_ilm->evaluate;
                $param->parent = $iassign_ilm->parent;
            }
        }
        return $param;
    }

    static function new_ilm($param) {
        global $DB;

        $newentry = new stdClass();
        $newentry->name = $param->name;
        $newentry->url = $param->url;
        $newentry->description = $param->description;
        $newentry->extension = $param->extension;
        $newentry->file_jar = $param->name;
        $newentry->file_class = $param->file_class;
        $newentry->width = $param->width;
        $newentry->heigth = $param->heigth;
        $newentry->enable = $param->enable;
        $newentry->timemodified = $param->timemodified;
        $newentry->author = $param->author;
        $newentry->timecreated = $param->timecreated;
        $newentry->evaluate = $param->evaluate;
        $newentry->parent = $param->parent;

        $newentry->id = $DB->insert_record("iassign_ilm", $newentry);
        if (!$newentry->id) {
            print_error('error_add_ilm', 'iassign');
        } else {

            $file_jar = new_file_ilm($param->file);
            $upentry = new stdClass();
            $upentry->id = $newentry->id;
            if (!is_null($file_jar))
                $upentry->file_jar = $file_jar;
            else
                $upentry->file_jar = $newentry->id;
            if (!$DB->update_record("iassign_ilm", $upentry))
                error(get_string('error_add_ilm', 'iassign'));
        }
    }

    function new_file_ilm($itemid) {
        global $CFG, $USER;

        $contextuser = get_context_instance(CONTEXT_USER, $USER->id);
        $fs = get_file_storage();
        $files_ilm = $fs->get_area_files($contextuser->id, 'user', 'draft', $itemid);

        $file_jar = array();
        if ($files_ilm) {
            foreach ($files_ilm as $value) {
                if ($value->get_filename() != '.') {
                    $filename = $value->get_filename();
                    $copy_to = $CFG->dirroot . '/mod/iassign/ilm/' . $filename;

                    while (file_exists($copy_to)) {
                        $filename = 'copy_' . $filename;
                        $copy_to = $CFG->dirroot . '/mod/iassign/ilm/' . $filename;
                    }
                    $value->copy_content_to($copy_to);
                    chmod($copy_to, 0755);
                    array_push($file_jar, $filename);
                }
            }
        }

        $delete_file = $fs->delete_area_files($contextuser->id, 'user', 'draft', $itemid);

        if (!empty($file_jar))
            return implode(",", $file_jar);
        else
            return null;
    }

    static function edit_ilm($param) {
        global $DB;

        $newentry = new stdClass();
        $newentry->id = $param->id;
        $newentry->name = $param->name;
        $newentry->url = $param->url;
        $newentry->description = $param->description;
        $newentry->extension = $param->extension;
        $newentry->file_class = $param->file_class;
        $newentry->width = $param->width;
        $newentry->heigth = $param->heigth;
        $newentry->enable = $param->enable;
        $newentry->timemodified = $param->timemodified;
        $newentry->evaluate = $param->evaluate;
        $newentry->file_jar = $param->file_jar;

        $file_jar = new_file_ilm($param->file);

        if (!is_null($file_jar))
            $newentry->file_jar = $newentry->file_jar . ',' . $file_jar;

        if (!$DB->update_record("iassign_ilm", $newentry)) {
            error(get_string('error_edit_ilm', 'iassign'));
        }
    }

    static function visible_ilm($ilm_id, $status) {
        global $DB;
        if ($status == 0)
            $visible = 1;
        else
            $visible = 0;
        $newentry = new stdClass();
        $newentry->id = $ilm_id;
        $newentry->enable = $visible;

        if (!$DB->update_record("iassign_ilm", $newentry))
            error(get_string('error_edit_ilm', 'iassign'));
    }

    static function view_ilm($ilmid) {
        global $CFG, $DB;

        $url = new moodle_url('/admin/settings.php', array('section' => 'modsettingiassign'));
        $iassign_ilm = $DB->get_records_sql("SELECT s.*
              FROM {$CFG->prefix}iassign_ilm s
              WHERE s.id = '$ilmid'");

        $str = "";
        $str.= '<table id="outlinetable" class="generaltable boxaligncenter" cellpadding="5" width="100%" >' . chr(13);

        if ($iassign_ilm) {
            foreach ($iassign_ilm as $ilm) {
                $iassign_statement = $DB->get_records('iassign_statement', array("iassign_ilmid" => $ilm->id));
                if ($iassign_statement)
                    $total = count($iassign_statement);
                else
                    $total = 0;

                $str .='<tr>';
                $str.= '<td class=\'cell c0 actvity\' width=60%><strong>' . get_string('name_ilm', 'iassign') . ':</strong>&nbsp;' . $ilm->name . '</td>' . chr(13);
                $str.= '<td class=\'cell c0 actvity\' ><strong>' . get_string('activities', 'iassign') . ':</strong>&nbsp;' . $total . '</td>' . chr(13);
                $str .= '<td class=\'cell c0 actvity\' > <input type=button value="' . get_string('close', 'iassign') . '"  onclick="javascript:window.close ();"></td>' . chr(13);
                $str .='</tr>';
                $str .='<tr><td colspan=3>';
                $str.= '<table width="100%">';
                $str .='<tr><td colspan=3><strong>' . get_string('description', 'iassign') . ':</strong>&nbsp;' . $ilm->description . '</td></tr>';
                $str .='<tr><td colspan=3><strong>' . get_string('file_jar', 'iassign') . '</strong>&nbsp;' . $ilm->file_jar . '</td></tr>';
                $str .='<tr><td ><strong>' . get_string('file_class', 'iassign') . ':</strong>&nbsp;' . $ilm->file_class . '</td>';
                $str .='<td colspan=2><strong>' . get_string('url_ilm', 'iassign') . ':</strong>&nbsp;<a href="' . $ilm->url . '">' . $ilm->url . '</a></td></tr>';
                $str .='<tr><td width="50%"><strong>' . get_string('extension', 'iassign') . ':</strong>&nbsp;' . $ilm->extension . '</td>';
                $str .='<td width="50%"><strong>' . get_string('width', 'iassign') . ':</strong>&nbsp;' . $ilm->width;
                $str .='&nbsp;&nbsp;<strong>' . get_string('heigth', 'iassign') . ':</strong>&nbsp;' . $ilm->heigth . '</td></tr>';
                if ($ilm->evaluate == 1)
                    $evaluate = get_string('yes', 'iassign');
                else
                    $evaluate = get_string('no', 'iassign');

                $str .='<tr><td width="50%"><strong>' . get_string('evaluate', 'iassign') . ':</strong>&nbsp;' . $evaluate . '</td>';
                if ($ilm->enable == 1)
                    $enable = get_string('yes', 'iassign');
                else
                    $enable = get_string('no', 'iassign');
                $str .='<td width="50%"><strong>' . get_string('enable', 'iassign') . ':</strong>&nbsp;' . $enable . '</td></tr>';
                $str.= '</table>';
                $str .='</td></tr>';
            }

            if (!empty($ilm->file_jar)) {
                $lang = substr(current_language(), 0, 2);
                $str .= '<tr><th colspan=3 class="header c1">' . get_string('area_test_lms', 'iassign') . '</th></tr>' . chr(13);
                $str .='<tr class=\'cell c0 actvity\'><td  colspan=3 align=center>';

                $str.= '
            <applet name="iLM" codebase="' . $CFG->wwwroot . '/mod/iassign/ilm/" code="' . $ilm->file_class . '" archive="' . $ilm->file_jar . '" width="' . $ilm->width . '" height="' . $ilm->heigth . '" vspace=10 hspace=10>
            <param name="MA_PARAM_notSEND" value="true"/>
            <param name="lang" value="' . $lang . '"/>
            </applet> <br/>';
            } else {

                $str .='<tr class=\'cell c0 actvity\'>';
                $str .='<td colspan=2 align=center>' . get_string('null_file', 'iassign') . '</td>';
                $str .='<td align=center><a href="' . $url . '</a></td>';
                $str .= '</tr>';
            }
            $str .= '</td></tr>';
        }

        $str .= '</table>';

        return $str;
    }

}

/**
 * 
 */
class ilm_manager {

    var $id; //course id
    var $url;
    var $from;

    /**
     * Constructor for the base ilm_manager class
     */
    function ilm_manager($id, $url, $from) {
        $this->id = $id; //course id
        $this->url = $url;
        $this->from = $from;
    }

    /**
     * iLM from creating online editor
     */
    function ilm_editor_new() {
        global $USER, $CFG, $COURSE, $DB, $OUTPUT, $PAGE;
        $iassignid = optional_param('iassignid', NULL, PARAM_INT); // iAssign ID
        $itemidfile = optional_param('itemidfile', NULL, PARAM_TEXT);
        $iassign = $DB->get_record("iassign_ilm", array("id" => $iassignid));
        $context = get_context_instance(CONTEXT_COURSE, $this->id);
        //verifica se o arquivo jar está cadastrado no BD
        if (!$iassign) {
            echo $OUTPUT->notification(get_string('error_confirms_ilm', 'iassign'), 'notifysuccess');
            die;
        }

        //verifica se o arquivo jar existe
        $pathfilejar = $CFG->dirroot . "/mod/iassign/ilm/" . $iassign->file_jar;
        if (!file_exists($pathfilejar)) {
            echo $OUTPUT->notification(get_string('error_confirms_jar', 'iassign'), 'notifysuccess');
            return false;
        }

        $extension = $iassign->extension;
        if (!empty($_POST)) {
            $string = $_POST['MA_POST_Archive'];
            $filename = optional_param('filename', NULL, PARAM_TEXT);
            $filename = $this->format_filename($filename);
            $arrayfilename = explode(".", $filename);
            if (count($arrayfilename) == 1)
                $filename = $arrayfilename[0] . '.' . $extension;
            $this->write_file_iassign($string, $filename, $itemidfile);
            die;
        }
        else {
            $fs = get_file_storage();
            $files = $fs->get_area_files($context->id, 'mod_iassign', 'ilm');
            $files_array = '';
            foreach ($files as $value) {
                if ($value->get_filename() != ".")
                    $files_array .="'" . $value->get_filename() . "',";
            }

            $files_array .="''";

            $file = null;
            $ia_content = "";
            $filename = "";
            $file_class = $iassign->file_class;
            $file_jar = $iassign->file_jar;
            $width = $iassign->width;
            $heigth = $iassign->heigth;
            $notSEND = true;
            $lang = substr(current_language(), 0, 2);
            $error_files_exists = get_string('error_file_exists', 'iassign');

            $output =
                    "<script type='text/javascript'>
             //<![CDATA[
             function submit_MA_Answer () {
               var docForm = document.formEnvio;
               var resposta_exerc = new Array(3);
               var valor_resposta = new Array(3);
               var sessao = new Array(3);
               var doc = document.iLM;
               resposta_exerc[0] = doc.getAnswer();
               valor_resposta[0] = doc.getEvaluation();
               docForm.MA_POST_Value.value = valor_resposta[0];
               docForm.MA_POST_Archive.value = resposta_exerc[0];
               var files = new Array(" . $files_array . ");
               var filename=docForm.filename.value+'.'+'$extension';

               if(resposta_exerc[0] == -1){
                 alert('" . get_string('error_null_iassign', 'iassign') . "');
                 return false\n;
               }
               else{
                if(docForm.filename.value==''){
                  alert('" . get_string('error_file_null_iassign', 'iassign') . "');
                  return false;
                }
              }
              for (i=0;i<files.length;i++){
                if(files[i]==docForm.filename.value || files[i]==filename){
                  alert('" . $error_files_exists . "');
                  return false;\n
                }
              }
              docForm.submit();
              return true;
            }
            //]]>
            </script>";

            $output .="
            <form name='formEnvio' id='formEnvio' method='post' enctype='multipart/form-data'> ";
            $output .= $OUTPUT->box_start();
            $output .="
              <table width='100%'>
              <tr><td>" . get_string('label_file_iassign', 'iassign') . " <input type='text' name='filename' size=50/>
                <input type=button value='" . get_string('label_write_iassign', 'iassign') . "' title='' onclick='submit_MA_Answer();'/></td>
              <td><input type=button value='" . get_string('close', 'iassign') . "' title='' onclick='javascript:history.back();'/></td>
              </tr>
              </table>";

            $output .= $OUTPUT->box_end();
            $output .="            
            <center>
            <applet name='iLM' codebase='../../mod/iassign/ilm/' code='$file_class' archive='$file_jar' width='$width' height='$heigth' vspace=10 hspace=10>
              <param name='MA_PARAM_PropositionURL' value='true'/>
              <param name='MA_PARAM_notSEND' value='$notSEND'/>
              <param name='lang' value='$lang'/>
            </applet>
            <input type='hidden' name='MA_POST_Archive' value='" . $ia_content . "'>
            <input type='hidden' name='MA_POST_Value'>
            </center>
            </form>";

            $title = get_string('title_editor_iassign', 'iassign');
            $PAGE->set_pagelayout('embedded');
            $PAGE->set_title($title);
            $PAGE->set_cacheable(false);
            echo $OUTPUT->header();
            echo $OUTPUT->heading($title);
            echo $output;
            echo $OUTPUT->footer();
        }
        die;
    }

    /**
     * iLM from creating online editor
     */
    function ilm_editor_update() {
        global $USER, $CFG, $COURSE, $DB, $OUTPUT, $PAGE;

        // $courseid = optional_param('courseid', 0, PARAM_INT); // Course ID
        $iassignid = optional_param('iassignid', NULL, PARAM_INT); // iAssign ID
        $fileid = optional_param('fileid', NULL, PARAM_TEXT);


        $iassign = $DB->get_record("iassign_ilm", array("id" => $iassignid));
        //verifica se o arquivo jar está cadastrado no BD
        if (!$iassign) {
            echo $OUTPUT->notification(get_string('error_confirms_ilm', 'iassign'), 'notifysuccess');
            die;
        }

        //verifica se o arquivo jar existe
        $pathfilejar = $CFG->dirroot . "/mod/iassign/ilm/" . $iassign->file_jar;
        if (!file_exists($pathfilejar)) {
            echo $OUTPUT->notification(get_string('error_confirms_jar', 'iassign'), 'notifysuccess');
            return false;
        }

        $fs = get_file_storage();
        $file = $fs->get_file_by_id($fileid);
        if ($file) {
            $filename = $this->format_filename($file->get_filename());
            $ia_content = $file->get_content();
        } else {
            echo $OUTPUT->notification(get_string('error_view_ilm', 'iassign'), 'notifysuccess');
            return false;
        }


        if (!empty($_POST)) {
            $string = $_POST['MA_POST_Archive'];
            $this->update_file_iassign($string, $filename, $fileid);
            die;
        } else {


            $file_class = $iassign->file_class;
            $file_jar = $iassign->file_jar;
            $width = $iassign->width;
            $heigth = $iassign->heigth;
            $notSEND = false;
            $lang = substr(current_language(), 0, 2);
            $iassignidssign = 0;

            if ($file) {
                $token = '';
                $view = -1;
                $end_file = $CFG->wwwroot . DIRECTORY_SEPARATOR . 'mod' . DIRECTORY_SEPARATOR . 'iassign' . DIRECTORY_SEPARATOR . 'ilm_security.php?id=' . $fileid . '&token=' . $token . '&view=' . $view;
            }
            else
                $end_file = '';

            $output =
                    "<script type='text/javascript'>
             //<![CDATA[
             function submit_MA_Answer () {
               var docForm = document.formEnvio;
               var resposta_exerc = new Array(3);
               var valor_resposta = new Array(3);
               var sessao = new Array(3);
               var doc = document.iLM;
               resposta_exerc[0] = doc.getAnswer();
               valor_resposta[0] = doc.getEvaluation();
               docForm.MA_POST_Value.value = valor_resposta[0];
               docForm.MA_POST_Archive.value = resposta_exerc[0];
               if(resposta_exerc[0] == -1){
                 alert('" . get_string('error_null_iassign', 'iassign') . "');
                 return false\n;
               }
               docForm.submit();
               return true;
             }
             //]]>
             </script>";

            $output .="
            <form name='formEnvio' id='formEnvio' method='post' enctype='multipart/form-data'> ";
            $output .= $OUTPUT->box_start();
            $output .="             
              <table width='100%'>
              <tr><td width='75%'>" . get_string('label_file_iassign', 'iassign') . "<b>$filename</b></td>
              <td align='right' width='25%'><input type=button value='" . get_string('label_write_iassign', 'iassign') . "' title='' onclick='submit_MA_Answer();'/>
              <input type='hidden' name='filename' value='$filename'/></td>
              <td>     <input type=button value='" . get_string('close', 'iassign') . "' title='' onclick='javascript:history.back();'/></td>
              </tr>              
              </table>";

            $output .= $OUTPUT->box_end();

            $output .="
              <center>
              <applet name='iLM' codebase='../../mod/iassign/ilm/' code='$file_class' archive='$file_jar' width='$width' height='$heigth' vspace=10 hspace=10>
                <param name='MA_PARAM_PropositionURL' value='true'/>
                <param name='MA_PARAM_notSEND' value='false'/>
                <param name='lang' value='$lang'/>
                <param name='MA_PARAM_Proposition' value='" . $end_file . "'>
              </applet>
              <input type='hidden' name='MA_POST_Archive' >
              <input type='hidden' name='MA_POST_Value'>
              </form>
              </center>";
            $title = get_string('title_editor_iassign', 'iassign');
            $PAGE->set_pagelayout('embedded');
            $PAGE->set_title($title);
            $PAGE->set_cacheable(false);
            echo $OUTPUT->header();
            echo $OUTPUT->heading($title);
            echo $output;
            echo $OUTPUT->footer();
        }
        die;
    }

    function write_file_iassign($string, $filename, $itemid) { //
        global $USER, $CFG, $COURSE;

        $fs = get_file_storage();
        $usercontext = get_context_instance(CONTEXT_USER, $USER->id);
        $fs->delete_area_files($usercontext->id, 'user', 'draft', $itemid);

        $context = get_context_instance(CONTEXT_COURSE, $this->id);

        $fileinfo = array(
            'contextid' => $context->id, // ID of course
            'component' => 'mod_iassign', // usually = table name
            'filearea' => 'ilm', // usually = table name
            'itemid' => $itemid, // usually = ID of row in table
            'filepath' => '/', // any path beginning and ending in /
            'userid' => $USER->id,
            'author' => $USER->firstname . ' ' . $USER->lastname,
            'license' => 'allrightsreserved', //allrightsreserved
            'filename' => $filename); // any filename
        // Create file containing text. '$string'
        $file_course = $fs->create_file_from_string($fileinfo, $string);
        $output = "<script type='text/javascript'>
                 //<![CDATA[
                 alert('" . get_string('sucess_write', 'iassign') . "');\n
                 window.location='" . $this->url . "';
                 //]]>
                 </script>";
        echo $output;
        die;
    }

    function update_file_iassign($string, $filename, $fileid) { //
        global $USER;
        $fs = get_file_storage();
        $file = $fs->get_file_by_id($fileid);
        $context = get_context_instance(CONTEXT_COURSE, $this->id);

        $fileinfo = array(
            'contextid' => $context->id, // ID of context
            'component' => 'mod_iassign', // usually = table name
            'filearea' => 'ilm', // usually = table name
            'itemid' => $file->get_itemid(), // usually = ID of row in table
            'filepath' => '/', // any path beginning and ending in /
            'userid' => $USER->id,
            'author' => $USER->firstname . ' ' . $USER->lastname,
            'license' => 'allrightsreserved', //allrightsreserved
            'filename' => $file->get_filename()); // any filename
        $file->delete();
        $file_course = $fs->create_file_from_string($fileinfo, $string);

        $output = "<script type='text/javascript'>
                 //<![CDATA[
                 alert('" . get_string('sucess_update', 'iassign') . "');\n
                 window.location='" . $this->url . "';
                 //]]>
                 </script>";
        echo $output;
        die;
    }

    function delete_file_ilm() { //
        global $USER, $COURSE;
        $fileid = optional_param('fileid', NULL, PARAM_TEXT);
        $fs = get_file_storage();
        $file = $fs->get_file_by_id($fileid);
        $file->delete();
        redirect(new moodle_url($this->url));
        die;
    }

    function duplicate_file_ilm() {
        global $USER, $COURSE;
        $fileid = optional_param('fileid', NULL, PARAM_INT);
        $filename = optional_param('filename', NULL, PARAM_TEXT);
        $fs = get_file_storage();
        $file = $fs->get_file_by_id($fileid);

        // $itemid=$file->get_itemid()*10;
        // Duplicate file
        $newfile = $fs->create_file_from_storedfile(array('filename' => $filename), $file);
        redirect(new moodle_url($this->url));
        die;
    }

    function add_ilm() {
        global $USER, $CFG, $COURSE, $DB;
        $fileid = optional_param('fileid', NULL, PARAM_INT);
        $filename = optional_param('filename', NULL, PARAM_TEXT);
        $output = "<script type='text/javascript'>
                 //<![CDATA[
                 window.opener.document.forms['mform1'].file.value='$fileid';
                 window.opener.document.forms['mform1'].filename.value='$filename';
                 window.close();
                 //]]>
                 </script>";
        echo $output;
        die();
    }

    /**
     * iLM from creating online editor
     */
    function preview_ilm() {
        global $USER, $CFG, $COURSE, $DB;

        $fileid = optional_param('fileid', NULL, PARAM_TEXT); // iAssign ID
        $tag_filter = '<p>&lt;ia toolbar=&lt;iadisable width=600 height=400&gt;' . $fileid . '&lt;/ia&gt;</p>';

        $javascript = "<script type='text/javascript'>
               //<![CDATA[
               function submit_close() {
                 window.opener.location.reload();
                 window.close();
                }
                //]]>
                </script>";

        $html = "
        <form name='formEnvio' id='formEnvio' method='post' enctype='multipart/form-data'>
        <table border='1'>
        <tr><td><p>&lt;ia toolbar=disable width=600 height=400&gt; $fileid &lt;/ia&gt; </p></td></tr>
        </table>
        <table>
        <tr><td align='center'><input type=button value='" . get_string('close', 'iassign') . "' title='' onclick='submit_close();'/></td></tr>
        </table>
        </form>";

        $string = "<p>&lt;ia toolbar=disable width=600 height=400&gt; $fileid &lt;/ia&gt; </p>";

        echo $javascript;
        echo filter_text($html);
        die;
    }

    /**
     * Format filename for save in data dir.
     * @author: Luciano Oliveira Borges
     * @last modification date 2011/11/16
     */
    function format_filename($text) {
        $text = htmlspecialchars(urldecode($text));
        $array1 = array("á", "à", "â", "ã", "ä", "é", "è", "ê", "ë", "í", "ì", "î", "ï", "ó", "ò", "ô", "õ", "ö", "ú", "ù", "û", "ü", "ç"
            , "Á", "À", "Â", "Ã", "Ä", "É", "È", "Ê", "Ë", "Í", "Ì", "Î", "Ï", "Ó", "Ò", "Ô", "Õ", "Ö", "Ú", "Ù", "Û", "Ü", "Ç"
            , "@", " ", "!", "?");
        $array2 = array("a", "a", "a", "a", "a", "e", "e", "e", "e", "i", "i", "i", "i", "o", "o", "o", "o", "o", "u", "u", "u", "u", "c"
            , "A", "A", "A", "A", "A", "E", "E", "E", "E", "I", "I", "I", "I", "O", "O", "O", "O", "O", "U", "U", "U", "U", "C"
            , "-", "_", "", "");
        return str_replace($array1, $array2, $text);
    }

    function view_files_ilm() {
        global $CFG, $DB, $USER;
        $context = get_context_instance(CONTEXT_COURSE, $this->id);
        $fs = get_file_storage();
        $files_course = $fs->get_area_files($context->id, 'mod_iassign', 'ilm', false, 'filename');
        $files_array = '';
        foreach ($files_course as $value) {
            if ($value->get_filename() != ".")
                $files_array .="'" . $value->get_filename() . "',";
        }
        $files_array .="''";
        $error_files_exists = get_string('error_file_exists', 'iassign');

        $code_javascript_ilm = "
   <script type='text/javascript'>
   //<![CDATA[
   function preview_ilm (fileid) {
    preview_ilm=window.open('$CFG->wwwroot/mod/iassign/ilm_manager.php?from=$this->from&id=$this->id&action=preview&fileid='+fileid,'','menubar=0,location=0,scrollbars,status,resizable,width=800 height=600');
   }

   function delete_ilm (fileid) {
     var resp;
     resp=confirm('" . get_string('delete_file', 'iassign') . "');
     if(resp){
        window.location='$CFG->wwwroot/mod/iassign/ilm_manager.php?from=$this->from&id=$this->id&action=delete&fileid='+fileid;
     }
   }
   function add_ilm_iassign (filename,fileid) {
     window.location='$CFG->wwwroot/mod/iassign/ilm_manager.php?from=$this->from&id=$this->id&action=addilm&fileid='+fileid+'&filename='+filename;
   }
   function duplicate_ilm (filename,fileid) {
     var filenamecopy;
     var i;
     var files = new Array($files_array);
     do {
        filenamecopy = prompt ('" . get_string('duplicate_file', 'iassign') . "',filename);
     } while (filenamecopy == '');
     if(filenamecopy == null)
       return false;\n
     else {
       for (i=0;i<files.length;i++){
           if(files[i]==filenamecopy){
             alert('$error_files_exists');
             return false;\n
           }
       }
       window.location='$CFG->wwwroot/mod/iassign/ilm_manager.php?from=$this->from&id=$this->id&action=duplicate&fileid='+fileid+'&filename='+filenamecopy;
     }
   }
   //]]>
   </script>";

        $output = "";

        foreach ($files_course as $value) {
            if ($value->get_filename() != '.') {

                $fileid = $value->get_id();

                //buscar fileid nas tabelas do iassig
                $list_filein_use = "";
                $iassign_statements = $DB->get_records("iassign_statement", array("file" => $value->get_id()));
                if ($iassign_statements) {
                    foreach ($iassign_statements as $iassign_statement) {
                        $list_filein_use .= $iassign_statement->name . "</br>";
                    }
                }

                $filename = $value->get_filename();
                $filepath = $value->get_filepath();

                $file = $filepath . $filename;

                $url = "{$CFG->wwwroot}/pluginfile.php/{$value->get_contextid()}/mod_iassign/ilm";
                $filename = $value->get_filename();
                $fileurl = $url . $value->get_filepath() . $value->get_itemid() . '/' . $filename;
                //echo html_writer::link($fileurl, $filename)."</br>";

                $author = $value->get_author();
                $fileid = $value->get_id();

                $extension = explode(".", $filename);
                $iassign_ilm = $DB->get_record("iassign_ilm", array('extension' => $extension[1], 'parent' => 0));

                if (empty($iassign_ilm))
                    $iassign_ilm->id = 1;
                $filename1 = '"' . $filename . '"';
                $link_add_ilm_iassign = "<a href='$CFG->wwwroot/mod/iassign/ilm_manager.php?from=$this->from&id=$this->id&action=addilm&fileid=$fileid&filename=$filename1'>" . icons::insert('add_ilm_iassign') . "</a>";

                if ($value->get_userid() == $USER->id) {
                    if ($iassign_statements) {
                        $link_edit = icons::insert('edit_iassign_disable');
                        $link_delete = icons::insert('delete_iassign_disable');
                    } else {
                        $link_edit = "<a href='$CFG->wwwroot/mod/iassign/ilm_manager.php?from=$this->from&id=$this->id&action=update&iassignid=$iassign_ilm->id&fileid=$fileid'>" . icons::insert('edit_iassign') . "</a>";
                        $link_delete = "<a href='' onclick='delete_ilm(" . $fileid . ");'>" . icons::insert('delete_iassign') . "</a>";
                    }
                    $link_duplicate = "<a href='' onclick='duplicate_ilm(" . $filename1 . "," . $fileid . ");'>" . icons::insert('duplicate_iassign') . "</a>";
                } else {
                    $link_edit = icons::insert('no_edit_iassign');
                    $link_delete = "";
                    $link_duplicate = "";
                }

                $tag_filter = '&nbsp; &lt;ia toolbar=disable width=600 height=400&gt;' . $fileid . '&lt;/ia&gt;'; //< (&lt;) - / (&frasl;) >(&gt;)
                $link_filter = "<a href='' onclick='preview_ilm(" . $fileid . ");'>" . icons::insert('preview_iassign') . "</a>";
                if ($this->from == 'iassign')
                    $output .="<tr><td width=15%>$link_edit &nbsp; $link_delete &nbsp;$link_duplicate&nbsp;$link_add_ilm_iassign</td>
                    <td width=30%><a href='$fileurl'>$filename</a></td>
                    <td width=15%>$author</td>
                    <td width=50%>$link_filter&nbsp;&nbsp;$tag_filter</td></tr>";
                else
                    $output .="<tr><td width=15%>$link_edit &nbsp; $link_delete &nbsp;$link_duplicate</td>
                    <td width=30%><a href='$fileurl'>$filename</a></td>
                    <td width=10%>$author</td>
                    <td width=50%>$link_filter&nbsp;&nbsp;$tag_filter</td></tr>";
            }
        }
        $html = "
            <table id='outlinetable' class='generaltable boxaligncenter' cellpadding='5' width='100%'>
            <tr><th class='header c1' width=15%>" . get_string('functions', 'iassign') . "</th>
                <th class='header c1' width=30%>" . get_string('file', 'iassign') . "</th>
                <th class='header c1' width=15%>" . get_string('author', 'iassign') . "</th>
                <th class='header c1' width=50%>" . get_string('tag_filter', 'iassign') . "</th></tr>
                $output
            </table>";

        echo $code_javascript_ilm;
        echo $html;
    }

}

?>
